# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Steuerkennzeichen(StrEnum):
    """
    Zur Kennzeichnung verschiedener Steuersätze und Verfahren.
    """

    UST_0 = "UST_0"  #: Keine Umsatzsteuer, bzw. nicht steuerbar.
    UST_19 = "UST_19"  #: Umsatzsteuer 19%
    UST_16 = "UST_16"  #: Umsatzsteuer 16%
    UST_7 = "UST_7"  #: Umsatzsteuer 7%
    VST_0 = "VST_0"  #: Keine Vorsteuer, bzw. nicht steuerbar.
    VST_19 = "VST_19"  #: Vorsteuer 19%
    VST_16 = "VST_16"  #: Vorsteuer 16%
    VST_7 = "VST_7"  #: Vorsteuer 7%
    RCV = "RCV"  #: Reverse Charge Verfahren (Umkehrung der Steuerpflicht)



"""
Contains AufAbschlagRegional and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

from ..enum.aufabschlagstyp import AufAbschlagstyp
from ..enum.aufabschlagsziel import AufAbschlagsziel
from ..enum.waehrungseinheit import Waehrungseinheit

# pylint: disable=R0801
from ..utils import postprocess_docstring
from .aufabschlagproort import AufAbschlagProOrt
from .com import COM
from .energiemix import Energiemix
from .preisgarantie import Preisgarantie
from .tarifeinschraenkung import Tarifeinschraenkung
from .vertragskonditionen import Vertragskonditionen
from .zeitraum import Zeitraum

# pylint: disable=too-few-public-methods, too-many-instance-attributes
# pylint: disable=no-name-in-module


@postprocess_docstring
class AufAbschlagRegional(COM):
    """
    Mit dieser Komponente können Auf- und Abschläge verschiedener Typen
    im Zusammenhang mit regionalen Gültigkeiten abgebildet werden.
    Hier sind auch die Auswirkungen auf verschiedene Tarifparameter modelliert,
    die sich durch die Auswahl eines Auf- oder Abschlags ergeben.

    .. raw:: html

        <object data="../_static/images/bo4e/com/AufAbschlagRegional.svg" type="image/svg+xml"></object>

    .. HINT::
        `AufAbschlagRegional JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/AufAbschlagRegional.json>`_

    """

    # required attributess
    #: Bezeichnung des Auf-/Abschlags
    bezeichnung: Optional[str] = None
    #: Werte für die gestaffelten Auf/Abschläge mit regionaler Eingrenzung
    betraege: Optional[list[AufAbschlagProOrt]] = None

    #: Beschreibung zum Auf-/Abschlag
    beschreibung: Optional[str] = None
    #:Typ des Aufabschlages (z.B. absolut oder prozentual)
    auf_abschlagstyp: Optional[AufAbschlagstyp] = None
    #: Diesem Preis oder den Kosten ist der Auf/Abschlag zugeordnet. Z.B. Arbeitspreis, Gesamtpreis etc.
    auf_abschlagsziel: Optional[AufAbschlagsziel] = None
    #: Gibt an in welcher Währungseinheit der Auf/Abschlag berechnet wird. Euro oder Ct.
    einheit: Optional[Waehrungseinheit] = None
    #: Internetseite, auf der die Informationen zum Auf-/Abschlag veröffentlicht sind
    website: Optional[str] = None
    #: Zusatzprodukte, die nur in Kombination mit diesem AufAbschlag erhältlich sind
    zusatzprodukte: Optional[list[str]] = None
    #: Voraussetzungen, die erfüllt sein müssen, damit dieser AufAbschlag zur Anwendung kommen kann
    voraussetzungen: Optional[list[str]] = None
    #: Durch die Anwendung des Auf/Abschlags kann eine Änderung des Tarifnamens auftreten.
    tarifnamensaenderungen: Optional[str] = None
    #: Zeitraum, in dem der Abschlag zur Anwendung kommen kann
    gueltigkeitszeitraum: Optional[Zeitraum] = None
    energiemixaenderung: Optional[Energiemix] = None
    """ Der Energiemix kann sich durch einen AufAbschlag ändern (z.B. zwei Cent Aufschlag für Ökostrom:
    Sollte dies der Fall sein, wird hier die neue Zusammensetzung des Energiemix angegeben."""
    vertagskonditionsaenderung: Optional[Vertragskonditionen] = None
    """ Änderungen in den Vertragskonditionen. Falls in dieser Komponenten angegeben,
    werden die Tarifparameter hiermit überschrieben."""
    garantieaenderung: Optional[Preisgarantie] = None
    """ Änderungen in den Garantievereinbarungen. Falls in dieser Komponenten angegeben,
    werden die Tarifparameter hiermit überschrieben."""
    einschraenkungsaenderung: Optional[Tarifeinschraenkung] = None
    """ Änderungen in den Einschränkungen zum Tarif. Falls in dieser Komponenten angegeben,
    werden die Tarifparameter hiermit überschrieben."""



import pytest

from bo4e import Gueltigkeitstyp, Regionskriterium, Regionskriteriumtyp
from tests.serialization_helper import assert_serialization_roundtrip


class TestRegionskriterium:
    @pytest.mark.parametrize(
        "regionskriterium",
        [
            pytest.param(
                Regionskriterium(
                    regionskriteriumtyp=Regionskriteriumtyp.REGELGEBIET_NAME,
                    gueltigkeitstyp=Gueltigkeitstyp.NICHT_IN,
                    wert="Was ist ein Regionskriterium?",
                ),
            ),
        ],
    )
    def test_regionskriterium_serialization_roundtrip(self, regionskriterium: Regionskriterium) -> None:
        """
        Test de-/serialisation of Regionskriterium with minimal attributes.
        """
        assert_serialization_roundtrip(regionskriterium)



"""
Contains Buendelvertrag class and corresponding marshmallow schema for de-/serialization
"""

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module
from typing import Annotated, Optional

import pydantic
from pydantic import Field

from ..com.unterschrift import Unterschrift
from ..com.vertragskonditionen import Vertragskonditionen
from ..enum.sparte import Sparte
from ..enum.typ import Typ
from ..enum.vertragsart import Vertragsart
from ..enum.vertragsstatus import Vertragsstatus
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt
from .geschaeftspartner import Geschaeftspartner
from .vertrag import Vertrag


@postprocess_docstring
class Buendelvertrag(Geschaeftsobjekt):
    """
    Abbildung eines Bündelvertrags.
    Es handelt sich hierbei um eine Liste von Einzelverträgen, die in einem Vertragsobjekt gebündelt sind.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Buendelvertrag.svg" type="image/svg+xml"></object>

    .. HINT::
        `Buendelvertrag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Buendelvertrag.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.BUENDELVERTRAG

    # pylint: disable=duplicate-code
    #: Eine im Verwendungskontext eindeutige Nummer für den Vertrag
    vertragsnummer: Optional[str] = None
    #: Hier ist festgelegt, um welche Art von Vertrag es sich handelt. Z.B. Netznutzungvertrag
    vertragsart: Optional[Vertragsart] = None
    #: Gibt den Status des Vertrages an
    vertragsstatus: Optional[Vertragsstatus] = None
    #: Unterscheidungsmöglichkeiten für die Sparte
    sparte: Optional[Sparte] = None
    #: Gibt an, wann der Vertrag beginnt (inklusiv)
    vertragsbeginn: Optional[pydantic.AwareDatetime] = None
    #: Gibt an, wann der Vertrag (voraussichtlich) endet oder beendet wurde (exklusiv)
    vertragsende: Optional[pydantic.AwareDatetime] = None
    #: Der "erstgenannte" Vertragspartner. In der Regel der Aussteller des Vertrags.
    #: Beispiel: "Vertrag zwischen Vertagspartner 1 ..."
    vertragspartner1: Optional[Geschaeftspartner] = None
    #: Der "zweitgenannte" Vertragspartner. In der Regel der Empfänger des Vertrags.
    #: Beispiel "Vertrag zwischen Vertagspartner 1 und Vertragspartner 2"
    vertragspartner2: Optional[Geschaeftspartner] = None

    #: Die Liste mit den Einzelverträgen zu den Abnahmestellen
    einzelvertraege: Optional[list[Vertrag]] = None
    #: Festlegungen zu Laufzeiten und Kündigungsfristen
    vertragskonditionen: Optional[list[Vertragskonditionen]] = None
    #: Unterzeichner des Vertragspartners1
    unterzeichnervp1: Optional[list[Unterschrift]] = None
    #: Unterzeichner des Vertragspartners2
    unterzeichnervp2: Optional[list[Unterschrift]] = None
    #: Beschreibung zum Vertrag
    beschreibung: Optional[str] = None



"""
Contains Person class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Annotated, Optional

import pydantic
from pydantic import Field

from ..com.adresse import Adresse
from ..com.kontaktweg import Kontaktweg
from ..com.zustaendigkeit import Zustaendigkeit
from ..enum.anrede import Anrede
from ..enum.titel import Titel
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

# pylint: disable=too-many-instance-attributes, too-few-public-methods, disable=duplicate-code


@postprocess_docstring
class Person(Geschaeftsobjekt):
    """
    Object containing information about a Person

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Person.svg" type="image/svg+xml"></object>

    .. HINT::
        `Person JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Person.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.PERSON
    #: Mögliche Anrede der Person
    anrede: Optional[Anrede] = None
    individuelle_anrede: Optional[str] = None
    """
    Im Falle einer nicht standardisierten Anrede kann hier eine frei definierbare Anrede vorgegeben werden.
    Beispiel: "Vereinsgemeinschaft", "Pfarrer", "Hochwürdigster Herr Abt".
    """
    #: Möglicher Titel der Person
    titel: Optional[Titel] = None
    #: Vorname der Person
    vorname: Optional[str] = None
    #: Nachname (Familienname) der Person
    nachname: Optional[str] = None
    #: Kontaktwege der Person
    kontaktwege: Optional[list[Kontaktweg]] = None
    #: Geburtsdatum der Person
    geburtsdatum: Optional[pydantic.AwareDatetime] = None
    #: Weitere Informationen zur Person
    kommentar: Optional[str] = None
    #: Liste der Abteilungen und Zuständigkeiten der Person
    zustaendigkeiten: Optional[list[Zustaendigkeit]] = None
    #: Adresse der Person, falls diese von der Adresse des Geschäftspartners abweicht
    adresse: Optional[Adresse] = None



"""
Contains PreisblattHardware class and corresponding marshmallow schema for de-/serialization
"""

from typing import Annotated, Optional

from pydantic import Field

from ..bo.geraet import Geraet
from ..enum.bilanzierungsmethode import Bilanzierungsmethode
from ..enum.dienstleistungstyp import Dienstleistungstyp
from ..enum.netzebene import Netzebene
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .preisblatt import Preisblatt

# pylint: disable=too-few-public-methods


@postprocess_docstring
class PreisblattHardware(Preisblatt):
    """
    Variante des Preisblattmodells zur Abbildung der Preise für zusätzliche Hardware

    .. raw:: html

        <object data="../_static/images/bo4e/bo/PreisblattHardware.svg" type="image/svg+xml"></object>

    .. HINT::
        `PreisblattHardware JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/PreisblattHardware.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.PREISBLATTHARDWARE
    # required attributes (additional to those of Preisblatt)
    #: Die Preise gelten für Marktlokationen der angebebenen Bilanzierungsmethode
    bilanzierungsmethode: Optional[Bilanzierungsmethode] = None
    #: Die Preise gelten für Messlokationen in der angebebenen Netzebene
    messebene: Optional[Netzebene] = None

    #: Der Preis betriftt das hier angegebene Gerät, z.B. ein Tarifschaltgerät
    basisgeraet: Optional[Geraet] = None

    #: Im Preis sind die hier angegebenen Dienstleistungen enthalten, z.B. Jährliche Ablesung
    inklusive_dienstleistungen: Optional[list[Dienstleistungstyp]] = None

    #: Im Preis sind die hier angegebenen Geräte mit enthalten, z.B. ein Wandler
    inklusive_geraete: Optional[list[Geraet]] = None



"""
Contains Tarifinfo class
and corresponding marshmallow schema for de-/serialization
"""

# pylint: disable=too-many-instance-attributes, too-few-public-methods
# pylint: disable=no-name-in-module
from typing import Annotated, Optional

import pydantic
from pydantic import Field

from ..com.energiemix import Energiemix
from ..com.vertragskonditionen import Vertragskonditionen
from ..com.zeitraum import Zeitraum
from ..enum.kundentyp import Kundentyp
from ..enum.registeranzahl import Registeranzahl
from ..enum.sparte import Sparte
from ..enum.tarifmerkmal import Tarifmerkmal
from ..enum.tariftyp import Tariftyp
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt
from .marktteilnehmer import Marktteilnehmer


@postprocess_docstring
class Tarifinfo(Geschaeftsobjekt):
    """
    Das BO Tarifinfo liefert die Merkmale, die einen Endkundentarif identifizierbar machen.
    Dieses BO dient als Basis für weitere BOs mit erweiterten Anwendungsmöglichkeiten.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Tarifinfo.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarifinfo JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Tarifinfo.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.TARIFINFO
    #: Name des Tarifs
    bezeichnung: Optional[str] = None
    #: Der Name des Marktpartners, der den Tarif anbietet
    anbietername: Optional[str] = None
    #: Strom oder Gas, etc.
    sparte: Optional[Sparte] = None
    #: Kundentypen für den der Tarif gilt, z.B. Privatkunden
    kundentypen: Optional[list[Kundentyp]] = None
    #: Die Art des Tarifes, z.B. Eintarif oder Mehrtarif
    registeranzahl: Optional[Registeranzahl] = None
    #: Hinweis auf den Tariftyp, z.B. Grundversorgung oder Sondertarif
    tariftyp: Optional[Tariftyp] = None
    #: Weitere Merkmale des Tarifs, z.B. Festpreis oder Vorkasse
    tarifmerkmale: Optional[list[Tarifmerkmal]] = None
    #: Der Marktteilnehmer (Lieferant), der diesen Tarif anbietet
    anbieter: Optional[Marktteilnehmer] = None

    #: Internetseite auf dem der Tarif zu finden ist
    website: Optional[str] = None
    #: Freitext
    bemerkung: Optional[str] = None

    #: Angabe, in welchem Zeitraum der Tarif gültig ist
    zeitliche_gueltigkeit: Optional[Zeitraum] = None
    #: Der Energiemix, der für diesen Tarif gilt
    energiemix: Optional[Energiemix] = None
    #: Mindestlaufzeiten und Kündigungsfristen zusammengefasst
    vertragskonditionen: Optional[Vertragskonditionen] = None
    anwendung_von: Optional[pydantic.AwareDatetime] = None
    """
    Angabe des inklusiven Zeitpunkts, ab dem der Tarif bzw. der Preis angewendet und abgerechnet wird,
    z.B. "2021-07-20T18:31:48Z"
    """



import pytest

from bo4e import Angebotsposition, Betrag, Menge, Preis
from tests.serialization_helper import assert_serialization_roundtrip


class TestAngebotsposition:
    @pytest.mark.parametrize(
        "ap",
        [
            pytest.param(
                Angebotsposition(
                    positionsmenge=Menge(),
                    positionspreis=Preis(),
                    positionskosten=Betrag(),
                    positionsbezeichnung="Beispielangebotsposition",
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, ap: Angebotsposition) -> None:
        """
        Test de-/serialisation of Regionskriterium with minimal attributes.
        """
        assert_serialization_roundtrip(ap)



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Titel(StrEnum):
    """
    Übersicht möglicher Titel, z.B. eines Geschäftspartners.
    """

    DR = "DR"  #: Doktor*In
    PROF = "PROF"  #: Professor*In
    PROF_DR = "PROF_DR"  #: Professor*In Dr.



import pytest

from bo4e import Kontaktart, Kontaktweg
from tests.serialization_helper import assert_serialization_roundtrip


class TestKontakt:
    @pytest.mark.parametrize(
        "kontakt",
        [
            pytest.param(
                Kontaktweg(
                    kontaktart=Kontaktart.E_MAIL,
                    beschreibung="irgendeine Spezifikation",
                    kontaktwert="test@bo4e.de",
                ),
                id="all attributes at first level",
            ),
        ],
    )
    def test_serialization_roundtrip(self, kontakt: Kontaktweg) -> None:
        """
        Test de-/serialisation of Kontakt.
        """

        assert_serialization_roundtrip(kontakt)



import pytest

from bo4e import (
    Adresse,
    Bilanzierungsmethode,
    Energierichtung,
    Geschaeftspartner,
    Kundentyp,
    Marktlokation,
    Netzebene,
    Sparte,
)
from tests.serialization_helper import assert_serialization_roundtrip


class TestMaLo:
    @pytest.mark.parametrize(
        "marktlokation",
        [
            pytest.param(
                Marktlokation(
                    marktlokations_id="51238696781",
                    sparte=Sparte.GAS,
                    lokationsadresse=Adresse(),
                    energierichtung=Energierichtung.EINSP,
                    bilanzierungsmethode=Bilanzierungsmethode.PAUSCHAL,
                    ist_unterbrechbar=True,  # optional attribute
                    netzebene=Netzebene.NSP,
                    endkunde=Geschaeftspartner(),
                    kundengruppen=[Kundentyp.GEWERBE, Kundentyp.PRIVAT],
                )
            )
        ],
    )
    def test_serialization_roundtrip(self, marktlokation: Marktlokation) -> None:
        """
        Test de-/serialisation of Marktlokation.
        """

        assert_serialization_roundtrip(marktlokation)



"""
Contains Kostenposition and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

import pydantic

from ..utils import postprocess_docstring
from .betrag import Betrag
from .com import COM
from .menge import Menge
from .preis import Preis

# pylint: disable=too-few-public-methods, too-many-instance-attributes


@postprocess_docstring
class Kostenposition(COM):
    """
    Diese Komponente wird zur Übertagung der Details zu einer Kostenposition verwendet.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Kostenposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Kostenposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Kostenposition.json>`_

    """

    #: Ein Titel für die Zeile. Hier kann z.B. der Netzbetreiber eingetragen werden, wenn es sich um Netzkosten handelt.
    positionstitel: Optional[str] = None

    betrag_kostenposition: Optional[Betrag] = None
    """Der errechnete Gesamtbetrag der Position als Ergebnis der Berechnung <Menge * Einzelpreis> oder
    <Einzelpreis / (Anzahl Tage Jahr) * zeitmenge>"""
    # todo: validate above calculation, see https://github.com/Hochfrequenz/BO4E-python/issues/282

    #: Bezeichnung für den Artikel für den die Kosten ermittelt wurden. Beispiel: Arbeitspreis HT
    artikelbezeichnung: Optional[str] = None

    #: Der Preis für eine Einheit. Beispiele: 5,8200 ct/kWh oder 55 €/Jahr.
    einzelpreis: Optional[Preis] = None

    #: inklusiver von-Zeitpunkt der Kostenzeitscheibe
    von: Optional[pydantic.AwareDatetime] = None
    #: exklusiver bis-Zeitpunkt der Kostenzeitscheibe
    bis: Optional[pydantic.AwareDatetime] = None

    #: Die Menge, die in die Kostenberechnung eingeflossen ist. Beispiel: 3.660 kWh
    menge: Optional[Menge] = None

    zeitmenge: Optional[Menge] = None
    """
    Wenn es einen zeitbasierten Preis gibt (z.B. €/Jahr), dann ist hier die Menge angegeben mit der die Kosten berechnet
    wurden. Z.B. 138 Tage.
    """

    #: Detaillierung des Artikels (optional). Beispiel: 'Drehstromzähler'
    artikeldetail: Optional[str] = None



from decimal import Decimal

import pytest

from bo4e import Energieherkunft, Erzeugungsart
from tests.serialization_helper import assert_serialization_roundtrip


class TestEnergieherkunft:
    @pytest.mark.parametrize(
        "energieherkunft",
        [
            pytest.param(
                Energieherkunft(
                    erzeugungsart=Erzeugungsart.BIOMASSE,
                    anteil_prozent=Decimal(25.5),
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(
        self,
        energieherkunft: Energieherkunft,
    ) -> None:
        """
        Test de-/serialisation of Energieherkunft with minimal attributes.
        """
        assert_serialization_roundtrip(energieherkunft)



"""
Contains Regionaltarif class and corresponding marshmallow schema for de-/serialization
"""

from typing import Annotated, Optional

import pydantic
from pydantic import Field

from ..com.regionalepreisgarantie import RegionalePreisgarantie
from ..com.regionaleraufabschlag import RegionalerAufAbschlag
from ..com.regionaletarifpreisposition import RegionaleTarifpreisposition
from ..com.tarifberechnungsparameter import Tarifberechnungsparameter
from ..com.tarifeinschraenkung import Tarifeinschraenkung
from ..enum.typ import Typ
from .tarifinfo import Tarifinfo

# pylint: disable=too-few-public-methods, empty-docstring
# pylint: disable=no-name-in-module


class Regionaltarif(Tarifinfo):
    #: Abbildung eines Tarifs mit regionaler Zuordnung von Preisen und Auf- und Abschlägen.
    """

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Regionaltarif.svg" type="image/svg+xml"></object>

    .. HINT::
        `Regionaltarif JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Regionaltarif.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.REGIONALTARIF
    #: Gibt an, wann der Preis zuletzt angepasst wurde
    preisstand: Optional[pydantic.AwareDatetime] = None
    #: Für die Berechnung der Kosten sind die hier abgebildeten Parameter heranzuziehen
    berechnungsparameter: Optional[Tarifberechnungsparameter] = None
    #: Die festgelegten Preise mit regionaler Eingrenzung, z.B. für Arbeitspreis, Grundpreis etc.
    tarifpreise: Optional[list[RegionaleTarifpreisposition]] = None

    #: Auf- und Abschläge auf die Preise oder Kosten mit regionaler Eingrenzung
    tarif_auf_abschlaege: Optional[list[RegionalerAufAbschlag]] = None
    #: Festlegung von Garantien für bestimmte Preisanteile
    preisgarantien: Optional[list[RegionalePreisgarantie]] = None
    #: Die Bedingungen und Einschränkungen unter denen ein Tarif angewendet werden kann
    tarifeinschraenkung: Optional[Tarifeinschraenkung] = None



from datetime import datetime, timezone

import pytest

from bo4e import (
    AufAbschlagRegional,
    Energiemix,
    Kundentyp,
    Marktteilnehmer,
    Preisgarantie,
    Registeranzahl,
    Sparte,
    Tarif,
    Tarifberechnungsparameter,
    Tarifeinschraenkung,
    Tarifmerkmal,
    TarifpreispositionProOrt,
    Tariftyp,
    Vertragskonditionen,
    Zeitraum,
)
from tests.serialization_helper import assert_serialization_roundtrip


class TestTarif:
    @pytest.mark.parametrize(
        "tarif",
        [
            pytest.param(
                Tarif(
                    preisstand=datetime(2022, 2, 1, 0, 0, 0, tzinfo=timezone.utc),
                    berechnungsparameter=Tarifberechnungsparameter(),
                    tarif_auf_abschlaege=[AufAbschlagRegional()],
                    tarifpreise=[TarifpreispositionProOrt()],
                    preisgarantie=Preisgarantie(),
                    tarifeinschraenkung=Tarifeinschraenkung(),
                    # below are the attributes of tarifinfo
                    bezeichnung="foo",
                    anbietername="der beste stromanbieter",
                    sparte=Sparte.STROM,
                    kundentypen=[Kundentyp.PRIVAT, Kundentyp.GEWERBE],
                    registeranzahl=Registeranzahl.MEHRTARIF,
                    tariftyp=Tariftyp.GRUND_ERSATZVERSORGUNG,
                    tarifmerkmale=[Tarifmerkmal.HEIZSTROM],
                    website="https://foo.inv",
                    bemerkung="super billig aber auch super dreckig",
                    vertragskonditionen=Vertragskonditionen(),
                    zeitliche_gueltigkeit=Zeitraum(),
                    energiemix=Energiemix(),
                    anbieter=Marktteilnehmer(),
                ),
                id="all attributes",
            ),
        ],
    )
    def test_serialization_roundtrip(self, tarif: Tarif) -> None:
        """
        Test de-/serialisation
        """
        assert_serialization_roundtrip(tarif)



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Medium(StrEnum):
    """
    Gibt ein physikalisches Medium an.
    """

    STROM = "STROM"  #: STROM
    GAS = "GAS"  #: GAS
    WASSER = "WASSER"  #: WASSER
    DAMPF = "DAMPF"  #: DAMPF



import pytest

from bo4e import Adresse, Dienstleistung, Geraet, Messlokation, Netzebene, Sparte, Zaehler
from tests.serialization_helper import assert_serialization_roundtrip


class TestMeLo:
    @pytest.mark.parametrize(
        "melo",
        [
            pytest.param(
                Messlokation(
                    messlokations_id="DE00056266802AO6G56M11SN51G21M24S",
                    sparte=Sparte.STROM,
                    netzebene_messung=Netzebene.MSP,
                    messgebietnr="664073",
                    geraete=[
                        Geraet(),
                        Geraet(),
                    ],
                    messdienstleistung=[
                        Dienstleistung(),
                        Dienstleistung(),
                    ],
                    messlokationszaehler=[Zaehler()],
                    grundzustaendiger_msb_codenr="9910125000002",
                    messadresse=Adresse(),
                )
            )
        ],
    )
    def test_serialization_roundtrip(self, melo: Messlokation) -> None:
        """
        Test de-/serialisation of Messlokation.
        """

        assert_serialization_roundtrip(melo)



"""
Contains TarifpreispositionProOrt class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

from ..utils import postprocess_docstring
from .com import COM
from .tarifpreisstaffelproort import TarifpreisstaffelProOrt

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module


@postprocess_docstring
class TarifpreispositionProOrt(COM):
    """
    Mit dieser Komponente können Tarifpreise verschiedener Typen abgebildet werden

    .. raw:: html

        <object data="../_static/images/bo4e/com/TarifpreispositionProOrt.svg" type="image/svg+xml"></object>

    .. HINT::
        `TarifpreispositionProOrt JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/TarifpreispositionProOrt.json>`_

    """

    #: Postleitzahl des Ortes für den der Preis gilt
    postleitzahl: Optional[str] = None
    #: Ort für den der Preis gilt
    ort: Optional[str] = None
    #: ene't-Netznummer des Netzes in dem der Preis gilt
    netznr: Optional[str] = None
    # Hier sind die Staffeln mit ihren Preisenangaben definiert
    preisstaffeln: Optional[list[TarifpreisstaffelProOrt]] = None
    # there are no optional attributes



"""
Contains Regionskriterium class and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

from ..enum.gueltigkeitstyp import Gueltigkeitstyp
from ..enum.regionskriteriumtyp import Regionskriteriumtyp
from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Regionskriterium(COM):
    """
    Komponente zur Abbildung eines Regionskriteriums

    .. raw:: html

        <object data="../_static/images/bo4e/com/Regionskriterium.svg" type="image/svg+xml"></object>

    .. HINT::
        `Regionskriterium JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Regionskriterium.json>`_

    """

    gueltigkeitstyp: Optional[
        Gueltigkeitstyp
    ] = None  #: Hier wird festgelegt, ob es sich um ein einschließendes oder ausschließendes Kriterium handelt.
    regionskriteriumtyp: Optional[
        Regionskriteriumtyp
    ] = None  #: Hier wird das Kriterium selbst angegeben, z.B. Bundesland.
    wert: Optional[str] = None
    """
    Der Wert, den das Kriterium annehmen kann, z.B. NRW.
    Im Falle des Regionskriteriumstyp BUNDESWEIT spielt dieser Wert keine Rolle.
    """



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Rechnungslegung(StrEnum):
    """
    Aufzählung der Möglichkeiten zur Rechnungslegung in Ausschreibungen.
    """

    MONATSRECHN = "MONATSRECHN"  #: monatsscharfe Rechnung
    ABSCHL_MONATSRECHN = "ABSCHL_MONATSRECHN"  #: Abschlag mit Monatsrechnung
    ABSCHL_JAHRESRECHN = "ABSCHL_JAHRESRECHN"  #: Abschlag mit Jahresrechnung
    MONATSRECHN_JAHRESRECHN = "MONATSRECHN_JAHRESRECHN"  #: Monatsrechnung mit Jahresrechnung
    VORKASSE = "VORKASSE"  #: Vorkasse



from decimal import Decimal

import pytest
from pydantic import ValidationError

from bo4e import Mengeneinheit, Preisstatus, Preistyp, Tarifpreis, Waehrungseinheit
from tests.serialization_helper import assert_serialization_roundtrip


class TestTarifpreis:
    def test_wrong_datatype(self) -> None:
        with pytest.raises(ValidationError) as excinfo:
            _ = Tarifpreis(
                wert="blubb",  # type: ignore[arg-type]
                einheit=Waehrungseinheit.EUR,
                bezugswert=Mengeneinheit.KWH,
                preistyp=Preistyp.ARBEITSPREIS_HT,
            )

        assert "1 validation error" in str(excinfo.value)
        assert "wert" in str(excinfo.value)
        assert "type=decimal_parsing" in str(excinfo.value)

    @pytest.mark.parametrize(
        "tarifpreis",
        [
            pytest.param(
                Tarifpreis(
                    wert=Decimal(3.50),
                    einheit=Waehrungseinheit.EUR,
                    bezugswert=Mengeneinheit.KWH,
                    status=Preisstatus.ENDGUELTIG,
                    preistyp=Preistyp.ARBEITSPREIS_HT,
                    beschreibung="Das ist ein HT Arbeitspreis",
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, tarifpreis: Tarifpreis) -> None:
        """
        Test de-/serialisation of Tarifpreis.
        """
        assert_serialization_roundtrip(tarifpreis)



"""
Contains Adresse class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

from ..enum.landescode import Landescode
from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-many-instance-attributes, too-few-public-methods


@postprocess_docstring
class Adresse(COM):
    """
    Contains an address that can be used for most purposes.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Adresse.svg" type="image/svg+xml"></object>

    .. HINT::
        `Adresse JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Adresse.json>`_

    """

    #: Die Postleitzahl; z.B: "41836"
    postleitzahl: Optional[str] = None
    #: Bezeichnung der Stadt; z.B. "Hückelhoven"
    ort: Optional[str] = None

    #: Bezeichnung des Ortsteils; z.B. "Mitte"
    ortsteil: Optional[str] = None
    #: Bezeichnung der Straße; z.B. "Weserstraße"
    strasse: Optional[str] = None
    #: Hausnummer inkl. Zusatz; z.B. "3", "4a"
    hausnummer: Optional[str] = None
    #: Im Falle einer Postfachadresse das Postfach; Damit werden Straße und Hausnummer nicht berücksichtigt
    postfach: Optional[str] = None
    #: Zusatzhinweis zum Auffinden der Adresse, z.B. "3. Stock linke Wohnung"
    adresszusatz: Optional[str] = None
    #: Im Falle einer c/o-Adresse steht in diesem Attribut die Anrede. Z.B. "c/o Veronica Hauptmieterin"
    co_ergaenzung: Optional[str] = None
    #: Offizieller ISO-Landescode
    landescode: Optional[Landescode] = Landescode.DE  # type:ignore



from decimal import Decimal

import pytest

from bo4e import (
    Gueltigkeitstyp,
    KriteriumWert,
    RegionaleGueltigkeit,
    RegionalePreisstaffel,
    Sigmoidparameter,
    Tarifregionskriterium,
)
from tests.serialization_helper import assert_serialization_roundtrip

example_regionale_preisstaffel = RegionalePreisstaffel(
    einheitspreis=Decimal(40.0),
    staffelgrenze_von=Decimal(12.5),
    staffelgrenze_bis=Decimal(25.0),
    sigmoidparameter=Sigmoidparameter(),
    regionale_gueltigkeit=RegionaleGueltigkeit(
        gueltigkeitstyp=Gueltigkeitstyp.NUR_IN,
        kriteriums_werte=[KriteriumWert(kriterium=Tarifregionskriterium.POSTLEITZAHL, wert="01069")],
    ),
)


class TestRegionalePreisstaffel:
    @pytest.mark.parametrize(
        "regionale_preisstaffel",
        [
            pytest.param(
                RegionalePreisstaffel(
                    einheitspreis=Decimal(40.0),
                    staffelgrenze_von=Decimal(12.5),
                    staffelgrenze_bis=Decimal(25.0),
                    sigmoidparameter=Sigmoidparameter(),
                    regionale_gueltigkeit=RegionaleGueltigkeit(
                        gueltigkeitstyp=Gueltigkeitstyp.NUR_IN,
                        kriteriums_werte=[KriteriumWert(kriterium=Tarifregionskriterium.POSTLEITZAHL, wert="01069")],
                    ),
                ),
                id="maximal attributes",
                # the messing sigmoidparameter is tested in the Preisstaffel tests
            ),
        ],
    )
    def test_serialization_roundtrip(
        self,
        regionale_preisstaffel: RegionalePreisstaffel,
    ) -> None:
        """
        Test de-/serialisation of RegionalePreisgarantie with maximal attributes.
        """
        assert_serialization_roundtrip(regionale_preisstaffel)



from decimal import Decimal

import pytest

from bo4e import (
    BDEWArtikelnummer,
    Bemessungsgroesse,
    Kalkulationsmethode,
    Leistungstyp,
    Mengeneinheit,
    Preisposition,
    Preisstaffel,
    Tarifzeit,
    Waehrungseinheit,
)
from tests.serialization_helper import assert_serialization_roundtrip


class TestPreisposition:
    @pytest.mark.parametrize(
        "preisposition",
        [
            pytest.param(
                Preisposition(
                    berechnungsmethode=Kalkulationsmethode.AP_TRANSPORT_ODER_VERTEILNETZ,
                    leistungstyp=Leistungstyp.SONSTIGER_PREIS,
                    leistungsbezeichnung="Foo",
                    preiseinheit=Waehrungseinheit.EUR,
                    bezugsgroesse=Mengeneinheit.KVARH,
                    zeitbasis=Mengeneinheit.HALBJAHR,
                    tarifzeit=Tarifzeit.TZ_HT,
                    bdew_artikelnummer=BDEWArtikelnummer.AUSGLEICHSENERGIE_UNTERDECKUNG,
                    zonungsgroesse=Bemessungsgroesse.BENUTZUNGSDAUER,
                    freimenge_blindarbeit=Decimal(50),  # %
                    freimenge_leistungsfaktor=Decimal(1.0),
                    preisstaffeln=[Preisstaffel()],
                    gruppenartikel_id="1-2-3",
                )
            ),
        ],
    )
    def test_serialization_roundtrip(self, preisposition: Preisposition) -> None:
        """
        Test de-/serialisation of Preisposition
        """
        assert_serialization_roundtrip(preisposition)



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Kundentyp(StrEnum):
    """
    Auflistung der Typen von Endkunden. Daraus kann das Verbrauchsprofil abgeleitet werden.
    """

    GEWERBE = "GEWERBE"  #: Gewerbekunden
    PRIVAT = "PRIVAT"  #: Privatkunden
    LANDWIRT = "LANDWIRT"  #: Landwirte
    SONSTIGE = "SONSTIGE"  #: Sonstige Endkunden
    HAUSHALT = "HAUSHALT"  #: Haushaltskunden
    DIREKTHEIZUNG = "DIREKTHEIZUNG"  #: Direktheizungen
    GEMEINSCHAFT_MFH = "GEMEINSCHAFT_MFH"  #: Gemeinschaftseinrichtungen von MFH
    KIRCHE = "KIRCHE"  #: Kirchen und caritative Einrichtungen
    KWK = "KWK"  #: KWK-Anlagen
    LADESAEULE = "LADESAEULE"  #: Ladesäulen
    BELEUCHTUNG_OEFFENTLICH = "BELEUCHTUNG_OEFFENTLICH"  #: Öffentliche Beleuchtungen
    BELEUCHTUNG_STRASSE = "BELEUCHTUNG_STRASSE"  #: Straßenbeleuchtungen
    SPEICHERHEIZUNG = "SPEICHERHEIZUNG"  #: Speicherheizungen
    UNTERBR_EINRICHTUNG = "UNTERBR_EINRICHTUNG"  #: Unterbrechbare Verbrauchseinrichtungen
    WAERMEPUMPE = "WAERMEPUMPE"  #: Wärmepumpen



"""
Contains Dienstleistung class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

from ..enum.dienstleistungstyp import Dienstleistungstyp
from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Dienstleistung(COM):
    """
    Abbildung einer abrechenbaren Dienstleistung.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Dienstleistung.svg" type="image/svg+xml"></object>

    .. HINT::
        `Dienstleistung JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Dienstleistung.json>`_

    """

    #: Kennzeichnung der Dienstleistung
    dienstleistungstyp: Optional[Dienstleistungstyp] = None
    #: Bezeichnung der Dienstleistung
    bezeichnung: Optional[str] = None



import pytest

from bo4e import Anrede, Organisationstyp, Titel
from bo4e.bo.geschaeftspartner import Geschaeftspartner
from bo4e.bo.person import Person
from bo4e.com.adresse import Adresse
from bo4e.enum.geschaeftspartnerrolle import Geschaeftspartnerrolle
from bo4e.zusatzattribut import ZusatzAttribut
from tests.serialization_helper import assert_serialization_roundtrip


class TestZusatzAttribut:
    @pytest.mark.parametrize(
        "geschaeftspartner",
        [
            pytest.param(
                Geschaeftspartner(
                    zusatz_attribute=[
                        ZusatzAttribut(name="SAP GP Nummer", wert="0123456789"),
                        ZusatzAttribut(name="Schufa-ID", wert="aksdlakoeuhn"),
                    ],
                    # just some dummy data to make the GP valid
                    ansprechpartner=[Person(), Person()],
                    anrede=Anrede.EHELEUTE,
                    individuelle_anrede="Künstler",
                    titel=Titel.PROF_DR,
                    vorname="Hans",
                    nachname="Müller-Schmidt",
                    organisationstyp=Organisationstyp.PRIVATPERSON,
                    geschaeftspartnerrollen=[Geschaeftspartnerrolle.KUNDE],
                    adresse=Adresse(
                        strasse="Am Geldspeicher",
                        hausnummer="17",
                        postleitzahl="88101",
                        ort="Entenhausen",
                    ),
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, geschaeftspartner: Geschaeftspartner) -> None:
        """
        Test de-/serialisation of Geschaeftspartner.
        """
        assert_serialization_roundtrip(geschaeftspartner)



"""
Contains Zaehlzeitregister class and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

from ..com.com import COM
from ..utils import postprocess_docstring

# pylint: disable=no-name-in-module
# pylint: disable=too-few-public-methods


@postprocess_docstring
class Zaehlzeitregister(COM):
    """
    Mit dieser Komponente werden Zählzeitregister modelliert. Ein Zählzeitregister beschreibt eine erweiterte Definition der Zählzeit
    in Bezug auf ein Register. Dabei werden alle Codes dazu vom Netzbetreiber vergeben.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zaehlzeitregister.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zaehlzeitregister JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Zaehlzeitregister.json>`_

    """

    zaehlzeit_definition: Optional[str] = None  #: Zählzeitdefinition
    zaehlzeit_register: Optional[str] = None  #: Zählzeitregister
    ist_schwachlastfaehig: Optional[bool] = None  #: Schwachlastfaehigkeit



"""
Contains Verbrauch and corresponding marshmallow schema for de-/serialization
"""

from decimal import Decimal
from typing import Optional

import pydantic

from ..enum.mengeneinheit import Mengeneinheit
from ..enum.messwertstatus import Messwertstatus
from ..enum.wertermittlungsverfahren import Wertermittlungsverfahren
from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module


@postprocess_docstring
class Verbrauch(COM):
    """
    Abbildung eines zeitlich abgegrenzten Verbrauchs

    .. raw:: html

        <object data="../_static/images/bo4e/com/Verbrauch.svg" type="image/svg+xml"></object>

    .. HINT::
        `Verbrauch JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Verbrauch.json>`_

    """

    #: Gibt an, ob es sich um eine PROGNOSE oder eine MESSUNG handelt
    wertermittlungsverfahren: Optional[Wertermittlungsverfahren] = None
    #: Die OBIS-Kennzahl für den Wert, die festlegt, welche Größe mit dem Stand gemeldet wird, z.B. '1-0:
    obis_kennzahl: Optional[str] = None
    #: Gibt den absoluten Wert der Menge an
    wert: Optional[Decimal] = None
    #: Gibt die Einheit zum jeweiligen Wert an
    einheit: Optional[Mengeneinheit] = None

    #: Inklusiver Beginn des Zeitraumes, für den der Verbrauch angegeben wird
    startdatum: Optional[pydantic.AwareDatetime] = None
    #: Exklusives Ende des Zeitraumes, für den der Verbrauch angegeben wird
    enddatum: Optional[pydantic.AwareDatetime] = None
    #: Messwertstatus includes the plausibility of the value
    messwertstatus: Optional[Messwertstatus] = None



import pytest

from bo4e import (
    KundengruppeKA,
    Marktteilnehmer,
    PreisblattKonzessionsabgabe,
    Preisposition,
    Preisstatus,
    Sparte,
    Zeitraum,
)
from tests.serialization_helper import assert_serialization_roundtrip


class TestPreisblatt:
    @pytest.mark.parametrize(
        "preisblatt_ka",
        [
            pytest.param(
                PreisblattKonzessionsabgabe(
                    bezeichnung="foo",
                    sparte=Sparte.STROM,
                    preisstatus=Preisstatus.ENDGUELTIG,
                    preispositionen=[Preisposition()],
                    gueltigkeit=Zeitraum(),
                    herausgeber=Marktteilnehmer(),
                    kundengruppe_k_a=KundengruppeKA.G_SONDERKUNDE,
                )
            ),
        ],
    )
    def test_serialization_roundtrip(self, preisblatt_ka: PreisblattKonzessionsabgabe) -> None:
        """
        Test de-/serialisation Preisblatt-Konzessionsabgabe.
        """
        assert_serialization_roundtrip(preisblatt_ka)



"""
Contains ExterenzeReferenz class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Any, Optional

from pydantic import BaseModel

# pylint: disable=too-few-public-methods
#
from .utils import postprocess_docstring


@postprocess_docstring
class ZusatzAttribut(BaseModel):
    """
    Viele Datenobjekte weisen in unterschiedlichen Systemen eine eindeutige ID (Kundennummer, GP-Nummer etc.) auf.
    Beim Austausch von Datenobjekten zwischen verschiedenen Systemen ist es daher hilfreich,
    sich die eindeutigen IDs der anzubindenden Systeme zu merken.

    .. raw:: html

        <object data="../_static/images/bo4e/com/ZusatzAttribut.svg" type="image/svg+xml"></object>

    .. HINT::
        `ZusatzAttribut JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/ZusatzAttribut.json>`_

    """

    name: Optional[str]  #: Bezeichnung der externen Referenz (z.B. "microservice xyz" oder "SAP CRM GP-Nummer")
    wert: Any  #: Wert der externen Referenz (z.B. "123456"; "4711")



"""
Contains Kontaktweg class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

from ..enum.kontaktart import Kontaktart
from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Kontaktweg(COM):
    """
    Die Komponente wird dazu verwendet, die Kontaktwege innerhalb des BOs Person darzustellen

    .. raw:: html

        <object data="../_static/images/bo4e/com/Kontakt.svg" type="image/svg+xml"></object>

    .. HINT::
        `Kontakt JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Kontakt.json>`_

    """

    #: Gibt die Kontaktart des Kontaktes an.
    kontaktart: Optional[Kontaktart] = None
    #: Spezifikation, beispielsweise "Durchwahl", "Sammelnummer" etc.
    beschreibung: Optional[str] = None
    #: Die Nummer oder E-Mail-Adresse.
    kontaktwert: Optional[str] = None
    #: Gibt an, ob es sich um den bevorzugten Kontaktweg handelt.
    ist_bevorzugter_kontaktweg: Optional[bool] = None



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Bilanzierungsmethode(StrEnum):
    """
    Mit dieser Aufzählung kann zwischen den Bilanzierungsmethoden bzw. -grundlagen unterschieden werden.
    """

    RLM = "RLM"  #: Registrierende Leistungsmessung
    SLP = "SLP"  #: Standard Lastprofil
    TLP_GEMEINSAM = "TLP_GEMEINSAM"  #: TLP gemeinsame Messung
    TLP_GETRENNT = "TLP_GETRENNT"  #: TLP getrennte Messung
    PAUSCHAL = "PAUSCHAL"  #: Pauschale Betrachtung (Band)



"""
Contains Preisblatt class
and corresponding marshmallow schema for de-/serialization
"""

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module
from typing import Annotated, Optional

from pydantic import Field

from ..com.preisposition import Preisposition
from ..com.zeitraum import Zeitraum
from ..enum.preisstatus import Preisstatus
from ..enum.sparte import Sparte
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt
from .marktteilnehmer import Marktteilnehmer


@postprocess_docstring
class Preisblatt(Geschaeftsobjekt):
    """
    Das allgemeine Modell zur Abbildung von Preisen;
    Davon abgeleitet können, über die Zuordnung identifizierender Merkmale, spezielle Preisblatt-Varianten modelliert
    werden.

    Die jeweiligen Sätze von Merkmalen sind in der Grafik ergänzt worden und stellen jeweils eine Ausprägung für die
    verschiedenen Anwendungsfälle der Preisblätter dar.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Preisblatt.svg" type="image/svg+xml"></object>

    .. HINT::
        `Preisblatt JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Preisblatt.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.PREISBLATT
    #: Eine Bezeichnung für das Preisblatt
    bezeichnung: Optional[str] = None
    #: Preisblatt gilt für angegebene Sparte
    sparte: Optional[Sparte] = None
    #: Merkmal, das anzeigt, ob es sich um vorläufige oder endgültige Preise handelt
    preisstatus: Optional[Preisstatus] = None
    #: Der Zeitraum für den der Preis festgelegt ist
    gueltigkeit: Optional[Zeitraum] = None
    #: Die einzelnen Positionen, die mit dem Preisblatt abgerechnet werden können. Z.B. Arbeitspreis, Grundpreis etc
    preispositionen: Optional[list[Preisposition]] = None
    #: Der Netzbetreiber, der die Preise veröffentlicht hat
    herausgeber: Optional[Marktteilnehmer] = None



import pytest

from bo4e import Betrag, Fremdkostenblock, Fremdkostenposition
from tests.serialization_helper import assert_serialization_roundtrip


class TestFremdkostenblock:
    @pytest.mark.parametrize(
        "fremdkostenblock",
        [
            pytest.param(
                Fremdkostenblock(
                    kostenblockbezeichnung="teststring",
                    kostenpositionen=[Fremdkostenposition()],
                    summe_kostenblock=Betrag(),
                ),
                id="all attributes at first level",
            ),
        ],
    )
    def test_serialization_roundtrip(self, fremdkostenblock: Fremdkostenblock) -> None:
        """
        Test de-/serialisation of Fremdkostenblock
        """
        assert_serialization_roundtrip(fremdkostenblock)



from decimal import Decimal

import pytest

from bo4e import Sigmoidparameter
from tests.serialization_helper import assert_serialization_roundtrip


class TestSigmoidparameter:
    @pytest.mark.parametrize(
        "sigmoidparameter",
        [
            pytest.param(
                Sigmoidparameter(
                    A=Decimal(1),
                    B=Decimal(2),
                    C=Decimal(3),
                    D=Decimal(4),
                ),
            ),
        ],
    )
    def test_sigmoidparameter_serialization_roundtrip(self, sigmoidparameter: Sigmoidparameter) -> None:
        """
        Test de-/serialisation of Sigmoidparameter with minimal attributes.
        """
        assert_serialization_roundtrip(sigmoidparameter)

    @pytest.mark.parametrize(
        "sigmoidparameter, leistung, expected_lp",
        [
            pytest.param(
                Sigmoidparameter(
                    A=Decimal(1),
                    B=Decimal(3),
                    C=Decimal(3),
                    D=Decimal(4),
                ),
                Decimal(3),
                Decimal(4.5),
            ),
        ],
    )
    def test_lp_calculation(self, sigmoidparameter: Sigmoidparameter, leistung: Decimal, expected_lp: Decimal) -> None:
        assert sigmoidparameter.calculate(leistung) == expected_lp



from datetime import datetime, timezone
from decimal import Decimal

import pytest

from bo4e import Registeranzahl, Sparte, Zaehler, Zaehlerauspraegung, Zaehlertyp, Zaehlwerk
from bo4e.zusatzattribut import ZusatzAttribut
from tests.serialization_helper import assert_serialization_roundtrip


class TestZaehler:
    @pytest.mark.parametrize(
        "zaehler",
        [
            pytest.param(
                Zaehler(
                    zaehlernummer="000111222",
                    sparte=Sparte.STROM,
                    zaehlerauspraegung=Zaehlerauspraegung.EINRICHTUNGSZAEHLER,
                    zaehlwerke=[Zaehlwerk()],
                    zaehlertyp=Zaehlertyp.DREHSTROMZAEHLER,
                    registeranzahl=Registeranzahl.ZWEITARIF,
                    zaehlerkonstante=Decimal(0.9),
                    eichung_bis=datetime(2022, 1, 1, 0, 0, 0).replace(tzinfo=timezone.utc),
                    zusatz_attribute=[ZusatzAttribut(name="zaehler im anderen system", wert="7890")],
                    letzte_eichung=datetime(2019, 6, 30, 0, 0, 0).replace(tzinfo=timezone.utc),
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, zaehler: Zaehler) -> None:
        """
        Test de-/serialisation of Zaehler.
        """
        assert_serialization_roundtrip(zaehler)



import pytest

from bo4e import AufAbschlag, AufAbschlagstyp, AufAbschlagsziel, Preisstaffel, Waehrungseinheit, Zeitraum
from tests.serialization_helper import assert_serialization_roundtrip


class TestAufAbschlag:
    @pytest.mark.parametrize(
        "aufabschlag",
        [
            pytest.param(
                AufAbschlag(
                    bezeichnung="foo",
                    beschreibung="bar",
                    auf_abschlagstyp=AufAbschlagstyp.ABSOLUT,
                    auf_abschlagsziel=AufAbschlagsziel.GESAMTPREIS,
                    einheit=Waehrungseinheit.EUR,
                    website="foo.bar",
                    gueltigkeitszeitraum=Zeitraum(),
                    staffeln=[Preisstaffel()],
                ),
                id="all attributes at first level",
            ),
        ],
    )
    def test_serialization_roundtrip(self, aufabschlag: AufAbschlag) -> None:
        """
        Test de-/serialisation of AufAbschlag with minimal attributes.
        """
        assert_serialization_roundtrip(aufabschlag)



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Tarifmerkmal(StrEnum):
    """
    Produktmerkmale im Zusammenhang mit der Tarifdefinition.
    """

    STANDARD = "STANDARD"  #: Standardprodukt
    VORKASSE = "VORKASSE"  #: Vorkassenprodukt
    PAKET = "PAKET"  #: Paketpreisprodukt
    KOMBI = "KOMBI"  #: Kombiprodukt
    FESTPREIS = "FESTPREIS"  #: Festpreisprodukt
    BAUSTROM = "BAUSTROM"  #: Baustromprodukt
    HAUSLICHT = "HAUSLICHT"  #: Hauslichtprodukt
    HEIZSTROM = "HEIZSTROM"  #: Heizstromprodukt
    ONLINE = "ONLINE"  #: Onlineprodukt



"""
Contains Rechnung class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Annotated, Optional

import pydantic
from pydantic import Field

from ..com.betrag import Betrag
from ..com.rechnungsposition import Rechnungsposition
from ..com.steuerbetrag import Steuerbetrag
from ..com.zeitraum import Zeitraum
from ..enum.netznutzungrechnungsart import NetznutzungRechnungsart
from ..enum.netznutzungrechnungstyp import NetznutzungRechnungstyp
from ..enum.rechnungsstatus import Rechnungsstatus
from ..enum.rechnungstyp import Rechnungstyp
from ..enum.sparte import Sparte
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt
from .geschaeftspartner import Geschaeftspartner
from .marktlokation import Marktlokation
from .messlokation import Messlokation

# pylint: disable=too-few-public-methods, too-many-instance-attributes


@postprocess_docstring
class Rechnung(Geschaeftsobjekt):
    """
    Modell für die Abbildung von Rechnungen und Netznutzungsrechnungen im Kontext der Energiewirtschaft;

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Rechnung.svg" type="image/svg+xml"></object>

    .. HINT::
        `Rechnung JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Rechnung.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.RECHNUNG
    ist_storno: Optional[bool] = None
    """
    Kennzeichnung, ob es sich um eine Stornorechnung handelt;
    im Falle "true" findet sich im Attribut "originalrechnungsnummer" die Nummer der Originalrechnung.
    """
    #: Eine im Verwendungskontext eindeutige Nummer für die Rechnung
    rechnungsnummer: Optional[str] = None
    #: Ausstellungsdatum der Rechnung
    rechnungsdatum: Optional[pydantic.AwareDatetime] = None
    #: Zu diesem Datum ist die Zahlung fällig
    faelligkeitsdatum: Optional[pydantic.AwareDatetime] = None
    #: Ein kontextbezogender Rechnungstyp, z.B. Netznutzungsrechnung
    rechnungstyp: Optional[Rechnungstyp] = None
    #: Der Zeitraum der zugrunde liegenden Lieferung zur Rechnung
    rechnungsperiode: Optional[Zeitraum] = None
    #: Der Aussteller der Rechnung, die Rollencodenummer kennt man über den im Geschäftspartner verlinkten Marktteilnehmer
    rechnungsersteller: Optional[Geschaeftspartner] = None
    #: Der Aussteller der Rechnung, die Rollencodenummer kennt man über den im Geschäftspartner verlinkten Marktteilnehmer
    rechnungsempfaenger: Optional[Geschaeftspartner] = None
    #: Die Summe der Nettobeträge der Rechnungsteile
    gesamtnetto: Optional[Betrag] = None
    #: Die Summe der Steuerbeträge der Rechnungsteile
    gesamtsteuer: Optional[Betrag] = None
    #: Die Summe aus Netto- und Steuerbetrag
    gesamtbrutto: Optional[Betrag] = None
    #: Der zu zahlende Betrag, der sich aus (gesamtbrutto - vorausbezahlt - rabattBrutto) ergibt
    zu_zahlen: Optional[Betrag] = None
    #: Die Rechnungspositionen
    rechnungspositionen: Optional[list[Rechnungsposition]] = None
    #: Bezeichnung für die vorliegende Rechnung
    rechnungstitel: Optional[str] = None
    #: Status der Rechnung zur Kennzeichnung des Bearbeitungsstandes
    rechnungsstatus: Optional[Rechnungsstatus] = None
    #: Im Falle einer Stornorechnung (storno = true) steht hier die Rechnungsnummer der stornierten Rechnung
    original_rechnungsnummer: Optional[str] = None
    #: Die Summe evtl. vorausgezahlter Beträge, z.B. Abschläge. Angabe als Bruttowert
    vorausgezahlt: Optional[Betrag] = None
    #: Gesamtrabatt auf den Bruttobetrag
    rabatt_brutto: Optional[Betrag] = None
    steuerbetraege: Optional[list[Steuerbetrag]] = None
    """
    Eine Liste mit Steuerbeträgen pro Steuerkennzeichen/Steuersatz;
    die Summe dieser Beträge ergibt den Wert für gesamtsteuer.
    """
    #: Sparte (Strom, Gas ...) für die die Rechnung ausgestellt ist
    sparte: Optional[Sparte] = None
    #: Aus der INVOIC entnommen, befüllt wenn es sich um eine Netznutzungsrechnung handelt
    netznutzungrechnungsart: Optional[NetznutzungRechnungsart] = None
    #: Aus der INVOIC entnommen, befüllt wenn es sich um eine Netznutzungsrechnung handelt
    netznutzungrechnungstyp: Optional[NetznutzungRechnungstyp] = None
    #: Kennzeichen, ob es sich um ein Original (true) oder eine Kopie handelt (false)
    ist_original: Optional[bool] = None
    #: Kennzeichen, ob es sich um eine simulierte Rechnung, z.B. zur Rechnungsprüfung handelt
    ist_simuliert: Optional[bool] = None
    #: Marktlokation, auf die sich die Rechnung bezieht
    marktlokation: Optional[Marktlokation] = None
    #: Messlokation, auf die sich die Rechnung bezieht
    messlokation: Optional[Messlokation] = None



"""
Contains Geraet class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Annotated, Optional

from pydantic import Field

from ..enum.geraeteklasse import Geraeteklasse
from ..enum.geraetetyp import Geraetetyp
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Geraet(Geschaeftsobjekt):
    """
    Mit diesem BO werden alle Geräte modelliert, die keine Zähler sind.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Geraet.svg" type="image/svg+xml"></object>

    .. HINT::
        `Geraet JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Geraet.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.GERAET

    #: Die auf dem Gerät aufgedruckte Nummer, die vom MSB vergeben wird.
    geraetenummer: Optional[str] = None
    #: Bezeichnung des Geräts
    bezeichnung: Optional[str] = None
    #: Die übergreifende Klasse eines Geräts, beispielsweise Wandler
    geraeteklasse: Optional[Geraeteklasse] = None
    #: Der speziellere Typ eines Gerätes, beispielsweise Stromwandler
    geraetetyp: Optional[Geraetetyp] = None



"""
Contains Tarif class and corresponding marshmallow schema for de-/serialization
"""

from typing import Annotated, Optional

import pydantic
from pydantic import Field

from ..com.aufabschlagregional import AufAbschlagRegional
from ..com.preisgarantie import Preisgarantie
from ..com.tarifberechnungsparameter import Tarifberechnungsparameter
from ..com.tarifeinschraenkung import Tarifeinschraenkung
from ..com.tarifpreispositionproort import TarifpreispositionProOrt
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .tarifinfo import Tarifinfo

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module


@postprocess_docstring
class Tarif(Tarifinfo):
    """
    Abbildung eines Tarifs mit regionaler Zuordnung von Preisen und Auf- und Abschlägen

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Tarif.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarif JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Tarif.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.TARIF
    #: Gibt an, wann der Preis zuletzt angepasst wurde
    preisstand: Optional[pydantic.AwareDatetime] = None
    #: Für die Berechnung der Kosten sind die hier abgebildeten Parameter heranzuziehen
    berechnungsparameter: Optional[Tarifberechnungsparameter] = None
    #: Die festgelegten Preise mit regionaler Eingrenzung z.B. für Arbeitspreis, Grundpreis etc.
    tarifpreise: Optional[list[TarifpreispositionProOrt]] = None

    #: Auf- und Abschläge auf die Preise oder Kosten mit regionaler Eingrenzung
    tarif_auf_abschlaege: Optional[list[AufAbschlagRegional]] = None
    # todo: fix inconsistency: RegionalerAufAbschlag vs. AufAbschlagRegional
    # https://github.com/Hochfrequenz/BO4E-python/issues/345

    #: Preisgarantie für diesen Tarif
    preisgarantie: Optional[Preisgarantie] = None
    # todo: fix inconsistency with regionaltarif https://github.com/Hochfrequenz/BO4E-python/issues/346
    #: Die Bedingungen und Einschränkungen unter denen ein Tarif angewendet werden kann
    tarifeinschraenkung: Optional[Tarifeinschraenkung] = None



"""
Contains Zeitreihenwert class
and corresponding marshmallow schema for de-/serialization
"""

from decimal import Decimal
from typing import Optional

from ..com.zeitspanne import Zeitspanne
from ..enum.messwertstatus import Messwertstatus
from ..enum.messwertstatuszusatz import Messwertstatuszusatz
from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Zeitreihenwert(COM):
    """
    Abbildung eines Zeitreihenwertes bestehend aus Zeitraum, Wert und Statusinformationen.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zeitreihenwert.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zeitreihenwert JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Zeitreihenwert.json>`_

    """

    zeitspanne: Optional[Zeitspanne] = None  #: Zeitespanne für das Messintervall

    wert: Optional[Decimal] = None  #: Der in der Zeitspanne gültige Wert.

    #: Der Status gibt an, wie der Wert zu interpretieren ist, z.B. in Berechnungen.
    status: Optional[Messwertstatus] = None

    #: Eine Zusatzinformation zum Status, beispielsweise ein Grund für einen fehlenden Wert.
    statuszusatz: Optional[Messwertstatuszusatz] = None



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Oekozertifikat(StrEnum):
    """
    Zertifikate für Ökostrom von verschiedenen Herausgebern.
    """

    CMS_EE01 = "CMS_EE01"  #: CMS_EE01
    CMS_EE02 = "CMS_EE02"  #: CMS_EE02
    EECS = "EECS"  #: EECS
    FRAUNHOFER = "FRAUNHOFER"  #: FRAUNHOFER
    BET = "BET"  #: BET
    KLIMA_INVEST = "KLIMA_INVEST"  #: KLIMA_INVEST
    LGA = "LGA"  #: LGA
    FREIBERG = "FREIBERG"  #: FREIBERG
    RECS = "RECS"  #: RECS
    REGS_EGL = "REGS_EGL"  #: REGS_EGL
    TUEV = "TUEV"  #: TUEV
    TUEV_HESSEN = "TUEV_HESSEN"  #: TUEV_HESSEN
    TUEV_NORD = "TUEV_NORD"  #: TUEV_NORD
    TUEV_RHEINLAND = "TUEV_RHEINLAND"  #: TUEV_RHEINLAND
    TUEV_SUED = "TUEV_SUED"  #: TUEV_SUED
    TUEV_SUED_EE01 = "TUEV_SUED_EE01"  #: TUEV_SUED_EE01
    TUEV_SUED_EE02 = "TUEV_SUED_EE02"  #: TUEV_SUED_EE02



import pytest

from bo4e import Energiemenge, Lokationstyp, Verbrauch
from tests.serialization_helper import assert_serialization_roundtrip


class TestEnergiemenge:
    @pytest.mark.parametrize(
        "energiemenge",
        [
            pytest.param(
                Energiemenge(
                    lokations_id="DE0123456789012345678901234567890",
                    lokationstyp=Lokationstyp.MELO,
                    energieverbrauch=[Verbrauch()],
                ),
                id="all attributes at first level",
            ),
        ],
    )
    def test_serialization_roundtrip(self, energiemenge: Energiemenge) -> None:
        assert_serialization_roundtrip(energiemenge)



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Tarifregionskriterium(StrEnum):
    """
    Mit diesen Kriterien können regionale Bereiche definiert werden.
    """

    NETZ_NUMMER = "NETZ_NUMMER"  #: Netznummer
    POSTLEITZAHL = "POSTLEITZAHL"  #: Postleitzahl
    ORT = "ORT"  #: Ort
    GRUNDVERSORGER_NUMMER = "GRUNDVERSORGER_NUMMER"  #: Nummer des Grundversorgers
    REGION = "REGION"  #: Referenz auf ein BO Region (URL)



"""
Contains Fremdkostenposition and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

from ..utils import postprocess_docstring
from .kostenposition import Kostenposition

# pylint: disable=too-few-public-methods, too-many-instance-attributes


@postprocess_docstring
class Fremdkostenposition(Kostenposition):
    """
    Eine Kostenposition im Bereich der Fremdkosten

    .. raw:: html

        <object data="../_static/images/bo4e/com/Fremdkostenposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Fremdkostenposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Fremdkostenposition.json>`_

    """

    # optional attributes (additional to those from Kostenposition)
    #: Der Name des Marktpartners, der die Preise festlegt, bzw. die Kosten in Rechnung stellt
    marktpartnername: Optional[str] = None

    #: Die Codenummer (z.B. BDEW-Codenummer) des Marktpartners, der die Preise festlegt / die Kosten in Rechnung stellt
    marktpartnercode: Optional[str] = None

    #: EIC-Code des Regel- oder Marktgebietes eingetragen. Z.B. '10YDE-EON------1' für die Regelzone TenneT
    gebietcode_eic: Optional[str] = None
    # todo: see issue https://github.com/Hochfrequenz/BO4E-python/issues/147 for EIC validation

    #: Link zum veröffentlichten Preisblatt
    link_preisblatt: Optional[str] = None



"""
Contains Standorteigenschaften class
and corresponding marshmallow schema for de-/serialization
"""

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module
from typing import Annotated, Optional

from pydantic import Field

from ..com.standorteigenschaftengas import StandorteigenschaftenGas
from ..com.standorteigenschaftenstrom import StandorteigenschaftenStrom
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt


@postprocess_docstring
class Standorteigenschaften(Geschaeftsobjekt):
    """
    Modelliert die regionalen und spartenspezifischen Eigenschaften einer gegebenen Adresse.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Standorteigenschaften.svg" type="image/svg+xml"></object>

    .. HINT::
        `Standorteigenschaften JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Standorteigenschaften.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.STANDORTEIGENSCHAFTEN
    #: Eigenschaften zur Sparte Strom
    eigenschaften_strom: Optional[list[StandorteigenschaftenStrom]] = None

    #: Eigenschaften zur Sparte Gas
    eigenschaften_gas: Optional[StandorteigenschaftenGas] = None



class TestImport:
    def test_importing_an_enum(self) -> None:
        """
        checks that importing an enum works
        """
        from bo4e import Vertragsstatus  # pylint: disable=import-outside-toplevel,unused-import

        assert True



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Vertragsart(StrEnum):
    """
    Aufzählung der Vertragsarten.
    """

    ENERGIELIEFERVERTRAG = "ENERGIELIEFERVERTRAG"  #: Energieliefervertrag
    NETZNUTZUNGSVERTRAG = "NETZNUTZUNGSVERTRAG"  #: Netznutzungsvertrag
    BILANZIERUNGSVERTRAG = "BILANZIERUNGSVERTRAG"  #: Bilanzierungsvertrag
    MESSSTELLENBETRIEBSVERTRAG = "MESSSTELLENBETRIEBSVERTRAG"  #: Messstellenabetriebsvertrag
    BUENDELVERTRAG = "BUENDELVERTRAG"  #: Bündelvertrag



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Gasqualitaet(StrEnum):
    """
    Unterscheidung für hoch- und niedrig-kalorisches Gas.
    """

    H_GAS = "H_GAS"  #: High Caloric Gas
    L_GAS = "L_GAS"  #: Low Caloric Gas



"""
Contains Tarifpreisblatt class and corresponding marshmallow schema for de-/serialization
"""

from typing import Annotated, Optional

import pydantic
from pydantic import Field

from ..com.aufabschlag import AufAbschlag
from ..com.preisgarantie import Preisgarantie
from ..com.tarifberechnungsparameter import Tarifberechnungsparameter
from ..com.tarifeinschraenkung import Tarifeinschraenkung
from ..com.tarifpreisposition import Tarifpreisposition
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .tarifinfo import Tarifinfo

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module


@postprocess_docstring
class Tarifpreisblatt(Tarifinfo):
    """
    Tarifinformation mit Preisen, Aufschlägen und Berechnungssystematik

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Tarifpreisblatt.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarifpreisblatt JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Tarifpreisblatt.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.TARIFPREISBLATT
    # required attributes (additional to those of Tarifinfo)
    #: Gibt an, wann der Preis zuletzt angepasst wurde
    preisstand: Optional[pydantic.AwareDatetime] = None
    #: Die festgelegten Preise, z.B. für Arbeitspreis, Grundpreis etc.
    tarifpreise: Optional[list[Tarifpreisposition]] = None
    #: Für die Berechnung der Kosten sind die hier abgebildeten Parameter heranzuziehen
    berechnungsparameter: Optional[Tarifberechnungsparameter] = None

    #: Die Bedingungen und Einschränkungen unter denen ein Tarif angewendet werden kann
    tarifeinschraenkung: Optional[Tarifeinschraenkung] = None
    #: Festlegung von Garantien für bestimmte Preisanteile
    preisgarantie: Optional[Preisgarantie] = None
    #: Auf- und Abschläge auf die Preise oder Kosten
    tarif_auf_abschlaege: Optional[list[AufAbschlag]] = None



from datetime import datetime, timezone

import pytest

from bo4e import BDEWArtikelnummer, Betrag, Menge, Mengeneinheit, Preis, Rechnungsposition, Steuerbetrag
from tests.serialization_helper import assert_serialization_roundtrip


class TestRechnungsposition:
    @pytest.mark.parametrize(
        "rechnungsposition",
        [
            pytest.param(
                Rechnungsposition(
                    positionsnummer=1,
                    lieferung_von=datetime(2021, 3, 15, tzinfo=timezone.utc),
                    lieferung_bis=datetime(2022, 3, 15, tzinfo=timezone.utc),
                    positionstext="Besonders wertvolle Rechnungsposition",
                    zeiteinheit=Mengeneinheit.JAHR,
                    artikelnummer=BDEWArtikelnummer.AUSGLEICHSENERGIE_UNTERDECKUNG,
                    lokations_id="51238696781",
                    positions_menge=Menge(),
                    zeitbezogene_menge=Menge(),
                    einzelpreis=Preis(),
                    teilsumme_netto=Betrag(),
                    teilrabatt_netto=Betrag(),
                    teilsumme_steuer=Steuerbetrag(),
                    artikel_id="7-8-9",
                ),
                id="maximal attributes",
            )
        ],
    )
    def test_serialization_roundtrip(self, rechnungsposition: Rechnungsposition) -> None:
        """
        Test de-/serialisation
        """
        assert_serialization_roundtrip(rechnungsposition)



"""
Contains Geokoordinaten class
and corresponding marshmallow schema for de-/serialization
"""

from decimal import Decimal
from typing import Optional

from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Geokoordinaten(COM):
    """
    This component provides the geo-coordinates for a location.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Geokoordinaten.svg" type="image/svg+xml"></object>

    .. HINT::
        `Geokoordinaten JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Geokoordinaten.json>`_

    """

    breitengrad: Optional[Decimal] = None
    laengengrad: Optional[Decimal] = None



from decimal import Decimal

import pytest

from bo4e import Betrag, Kostenblock, Kostenposition, Mengeneinheit, Preis, Preisstatus, Waehrungscode, Waehrungseinheit
from tests.serialization_helper import assert_serialization_roundtrip


class TestKostenblock:
    @pytest.mark.parametrize(
        "kostenblock",
        [
            pytest.param(
                Kostenblock(
                    kostenblockbezeichnung="Mein Kostenblock",
                    summe_kostenblock=Betrag(
                        waehrung=Waehrungscode.EUR,
                        wert=Decimal(12.5),
                    ),
                    kostenpositionen=[
                        Kostenposition(
                            positionstitel="Mudders kostenposition",
                            artikelbezeichnung="Dei Mudder ihr kostenposition",
                            einzelpreis=Preis(
                                wert=Decimal(3.50),
                                einheit=Waehrungseinheit.EUR,
                                bezugswert=Mengeneinheit.KWH,
                                status=Preisstatus.ENDGUELTIG,
                            ),
                            betrag_kostenposition=Betrag(
                                waehrung=Waehrungscode.EUR,
                                wert=Decimal(12.5),
                            ),
                        ),
                    ],
                ),
                id="all attributes at first level",
            ),
        ],
    )
    def test_serialization_roundtrip(self, kostenblock: Kostenblock) -> None:
        """
        Test de-/serialisation of kostenblock.
        """
        assert_serialization_roundtrip(kostenblock)



import pytest

from bo4e import Adresse, Anrede, Kontaktweg, Person, Titel, Zustaendigkeit
from tests.serialization_helper import assert_serialization_roundtrip


class TestPerson:
    @pytest.mark.parametrize(
        "person",
        [
            pytest.param(
                Person(
                    anrede=Anrede.EHELEUTE,
                    individuelle_anrede="Künstler",
                    titel=Titel.PROF_DR,
                    vorname="Hans",
                    nachname="Müller-Schmidt",
                    kontaktwege=[Kontaktweg(), Kontaktweg()],
                    kommentar="does this thing work?",
                    adresse=Adresse(),
                    zustaendigkeiten=[Zustaendigkeit()],
                ),
                id="all attributes at first level",  # = min + menge and betrag
            ),
        ],
    )
    def test_serialization_roundtrip(self, person: Person) -> None:
        """
        Test Test de-/serialisation roundtrip of Person
        """
        assert_serialization_roundtrip(person)



"""
Contains Zeitreihe class and corresponding marshmallow schema for de-/serialization
"""

# pylint: disable=too-few-public-methods, too-many-instance-attributes
# pylint: disable=no-name-in-module
from typing import Annotated, Optional

from pydantic import Field

from ..com.zeitreihenwert import Zeitreihenwert
from ..enum.medium import Medium
from ..enum.mengeneinheit import Mengeneinheit
from ..enum.messart import Messart
from ..enum.messgroesse import Messgroesse
from ..enum.typ import Typ
from ..enum.wertermittlungsverfahren import Wertermittlungsverfahren
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt


@postprocess_docstring
class Zeitreihe(Geschaeftsobjekt):
    """
    Abbildung einer allgemeinen Zeitreihe mit einem Wertvektor.
    Die Werte können mit wahlfreier zeitlicher Distanz im Vektor abgelegt sein.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Zeitreihe.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zeitreihe JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Zeitreihe.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.ZEITREIHE
    #: Bezeichnung für die Zeitreihe
    bezeichnung: Optional[str] = None
    #: Beschreibt, was gemessen wurde (z.B. Strom, Spannung, Wirkleistung, Scheinleistung)
    messgroesse: Optional[Messgroesse] = None
    #: Beschreibt die Art der Messung (z.B. aktueller Wert, mittlerer Wert, maximaler Wert)
    messart: Optional[Messart] = None
    #: Medium, das gemessen wurde (z.B. Wasser, Dampf, Strom, Gas)
    medium: Optional[Medium] = None
    #: Alle Werte in der Tabelle haben die Einheit, die hier angegeben ist
    einheit: Optional[Mengeneinheit] = None
    #: Hier liegen jeweils die Werte
    werte: Optional[list[Zeitreihenwert]] = None

    #: Beschreibt die Verwendung der Zeitreihe
    beschreibung: Optional[str] = None
    #: Version der Zeitreihe
    version: Optional[str] = None
    #: Kennzeichnung, wie die Werte entstanden sind, z.B. durch Messung
    wertherkunft: Optional[Wertermittlungsverfahren] = None



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Zaehlergroesse(StrEnum):
    """
    Auflistung möglicher Größen von Zählern
    """

    G2KOMMAP5 = "G2KOMMA5"  #: Gaszähler Größe G2.5
    G4 = "G4"  #: Gaszähler Größe G4
    G6 = "G6"  #: Gaszähler Größe G6
    G10 = "G10"  #: Gaszähler Größe G10
    G16 = "G16"  #: Gaszähler Größe G16
    G25 = "G25"  #: Gaszähler Größe G25
    G40 = "G40"  #: Gaszähler Größe G40
    G65 = "G65"  #: Gaszähler Größe G65
    G100 = "G100"  #: Gaszähler Größe G100
    G160 = "G160"  #: Gaszähler Größe G160
    G250 = "G250"  #: Gaszähler Größe G250
    G400 = "G400"  #: Gaszähler Größe G400
    G650 = "G650"  #: Gaszähler Größe G650
    G1000 = "G1000"  #: Gaszähler Größe G1000
    G1600 = "G1600"  #: Gaszähler Größe G1600
    G2500 = "G2500"  #: Gaszähler Größe G2500
    G4000 = "G4000"  #: Gaszähler Größe G4000
    G6500 = "G6500"  #: Gaszähler Größe G6500
    G10000 = "G10000"  #: Gaszähler Größe G10000
    G12500 = "G12500"  #: Gaszähler Größe G12500
    G16000 = "G16000"  #: Gaszähler Größe G16000



import pytest

from bo4e import Betrag, Kosten, Kostenblock, Kostenklasse, Zeitraum
from tests.serialization_helper import assert_serialization_roundtrip


class TestKosten:
    @pytest.mark.parametrize(
        "kosten",
        [
            pytest.param(
                Kosten(
                    kostenklasse=Kostenklasse.FREMDKOSTEN,
                    gueltigkeit=Zeitraum(),
                    kostenbloecke=[Kostenblock()],
                    summe_kosten=[Betrag()],
                ),
                id="all attributes at first level",
            )
        ],
    )
    def test_serialization_roundtrip(self, kosten: Kosten) -> None:
        """
        Test de-/serialisation of Kosten.
        """
        assert_serialization_roundtrip(kosten)



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Zaehlerauspraegung(StrEnum):
    """
    Gibt an, ob es sich um einen Einrichtungs- oder Zweirichtungszähler handelt.
    """

    EINRICHTUNGSZAEHLER = "EINRICHTUNGSZAEHLER"  #: Einrichtungszaehler
    ZWEIRICHTUNGSZAEHLER = "ZWEIRICHTUNGSZAEHLER"  #: Zweirichtungszaehler



"""
Contains Tarifpreis class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

from ..enum.preistyp import Preistyp
from ..utils import postprocess_docstring
from .preis import Preis

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Tarifpreis(Preis):
    """
    Abbildung eines Tarifpreises mit Preistyp und Beschreibung abgeleitet von COM Preis.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Tarifpreis.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarifpreis JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Tarifpreis.json>`_

    """

    #:  Angabe des Preistypes (z.B. Grundpreis)
    preistyp: Optional[Preistyp] = None

    #:  Beschreibung des Preises. Hier können z.B. Preisdetails angegeben sein, beispielsweise "Drehstromzähler".
    beschreibung: Optional[str] = None



"""
Contains Tarifkosten class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Annotated, Optional

from pydantic import Field

from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .kosten import Kosten
from .tarifinfo import Tarifinfo

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Tarifkosten(Tarifinfo):
    """
    Objekt zur Kommunikation von Kosten, die im Rahmen der Tarifanwendung entstehen

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Tarifkosten.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarifkosten JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Tarifkosten.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.TARIFKOSTEN
    kosten: Optional[Kosten] = None
    """
    Referenz (Link) zu einem Kostenobjekt, in dem die Kosten für die Anwendung
    des Tarifs auf eine Abnahmesituation berechnet wurden
    """

    # there are no optional attributes



from datetime import datetime, timezone

from bo4e.com.zeitspanne import Zeitspanne


class TestZeitspanne:
    def test_zeitspanne(self) -> None:
        """
        Test de-/serialisation of Zeitspanne
        """
        zeitspanne = Zeitspanne(
            start=datetime(2013, 5, 1, tzinfo=timezone.utc), ende=datetime(2022, 1, 28, tzinfo=timezone.utc)
        )

        json_string = zeitspanne.model_dump_json(by_alias=True)

        assert "2013-05-01T00:00:00Z" in json_string
        assert "2022-01-28T00:00:00Z" in json_string

        zeitspanne_deserialized = Zeitspanne.model_validate_json(json_string)
        assert zeitspanne_deserialized == zeitspanne



"""
contains the COM Zeitspanne
"""

from typing import Optional

import pydantic

from bo4e.com.com import COM

from ..utils import postprocess_docstring


@postprocess_docstring
class Zeitspanne(COM):
    """
    Eine Zeitspanne ist definiert aus Start und/oder Ende.
    Der Unterschied zur Menge (die auch zur Abbildung von Zeitmengen genutzt wird) ist, dass konkrete Start- und Endzeitpunkte angegeben werden.
    Die Zeitspanne ist aus dem COM Zeitraum hervorgegangen, das in Zeitspanne und Menge aufgeteilt wurde.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zeitspanne.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zeitspanne JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Zeitspanne.json>`_

    """

    start: Optional[pydantic.AwareDatetime] = None  #: inklusiver Beginn
    ende: Optional[pydantic.AwareDatetime] = None  #: exklusives Ende



from datetime import datetime, timezone

import pytest

from bo4e import (
    Ausschreibung,
    Ausschreibungslos,
    Ausschreibungsportal,
    Ausschreibungsstatus,
    Ausschreibungstyp,
    Geschaeftspartner,
    Zeitraum,
)
from tests.serialization_helper import assert_serialization_roundtrip


class TestAusschreibung:
    @pytest.mark.parametrize(
        "ausschreibung",
        [
            pytest.param(
                Ausschreibung(
                    ausschreibungsnummer="239230",
                    ausschreibungstyp=Ausschreibungstyp.PRIVATRECHTLICH,
                    ausschreibungsstatus=Ausschreibungsstatus.PHASE3,
                    ist_kostenpflichtig=True,
                    ausschreibungportal=Ausschreibungsportal.BMWI,
                    webseite="https://meineausschreibungswebsite.inv/",
                    veroeffentlichungszeitpunkt=datetime(2022, 1, 1, 0, 0, 0, tzinfo=timezone.utc),
                    abgabefrist=Zeitraum(),
                    bindefrist=Zeitraum(),
                    ausschreibender=Geschaeftspartner(),
                    lose=[Ausschreibungslos()],
                ),
                id="all attributes at first level",
            ),
        ],
    )
    def test_serialization_roundtrip(self, ausschreibung: Ausschreibung) -> None:
        """
        Test de-/serialisation of Ausschreibung.
        """
        assert_serialization_roundtrip(ausschreibung)



import pytest

from bo4e import Geschaeftspartner, Marktrolle, Marktteilnehmer, Rollencodetyp, Sparte
from tests.serialization_helper import assert_serialization_roundtrip


class TestMarktteilnehmer:
    @pytest.mark.parametrize(
        "marktteilnehmer",
        [
            pytest.param(
                Marktteilnehmer(
                    marktrolle=Marktrolle.DL,
                    rollencodenummer="9903916000000",
                    rollencodetyp=Rollencodetyp.BDEW,
                    sparte=Sparte.STROM,
                    makoadresse=["stringadressewarum", "as4adresse"],
                    geschaeftspartner=Geschaeftspartner(),
                )
            )
        ],
    )
    def test_serialization_roundtrip(self, marktteilnehmer: Marktteilnehmer) -> None:
        """
        Test de-/serialisation of Marktteilnehmer.
        """

        assert_serialization_roundtrip(marktteilnehmer)



"""
Contains Fremdkostenblock class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

from ..utils import postprocess_docstring
from .betrag import Betrag
from .com import COM
from .fremdkostenposition import Fremdkostenposition

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Fremdkostenblock(COM):
    """
    Komponente zur Abbildung eines Kostenblocks in den Fremdkosten

    .. raw:: html

        <object data="../_static/images/bo4e/com/Fremdkostenblock.svg" type="image/svg+xml"></object>

    .. HINT::
        `Fremdkostenblock JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Fremdkostenblock.json>`_

    """

    #: Bezeichnung für einen Kostenblock. Z.B. Netzkosten, Messkosten, Umlagen, etc.
    kostenblockbezeichnung: Optional[str] = None

    kostenpositionen: Optional[list[Fremdkostenposition]] = None
    """
    Hier sind die Details zu einer Kostenposition aufgeführt. Z.B.:
    Alliander Netz Heinsberg GmbH, 2018-02-01, 2019-01-01, Arbeitspreis HT, 3.660 kWh,
    5,8200 ct/kWh, 213,01 €
    """

    #: Die Summe aller Kostenpositionen dieses Blocks
    summe_kostenblock: Optional[Betrag] = None
    # todo: write validator fo this sum, see https://github.com/Hochfrequenz/BO4E-python/issues/324



# pylint: disable=missing-module-docstring,line-too-long
from bo4e.enum.strenum import StrEnum


class Tarifkalkulationsmethode(StrEnum):
    """
    Auflistung der verschiedenen Berechnungsmethoden für ein Preisblatt.
    """

    KEINE = "KEINE"  #: Es wird keine Berechnung durchgeführt, sondern lediglich die Menge mit dem Preis multipliziert.
    STAFFELN = "STAFFELN"  #: Staffelmodell, d.h. die Gesamtmenge wird in eine Staffel eingeordnet und für die gesamte Menge gilt der so ermittelte Preis
    ZONEN = "ZONEN"  #: Zonenmodell, d.h. die Gesamtmenge wird auf die Zonen aufgeteilt und für die Teilmengen gilt der jeweilige Preis der Zone.
    BESTABRECHNUNG_STAFFEL = "BESTABRECHNUNG_STAFFEL"  #: Bestabrechnung innerhalb der Staffelung
    PAKETPREIS = "PAKETPREIS"  #: Preis für ein Paket (eine Menge).



from datetime import datetime, timezone

import pytest

from bo4e import Angebot, Geschaeftspartner, Person, Sparte
from tests.serialization_helper import assert_serialization_roundtrip


class TestAngebot:
    @pytest.mark.parametrize(
        "angebot",
        [
            pytest.param(
                Angebot(
                    angebotsnummer="271828",
                    anfragereferenz="foo",
                    angebotsdatum=datetime(2020, 1, 1, tzinfo=timezone.utc),
                    sparte=Sparte.GAS,
                    bindefrist=datetime(2019, 3, 2, tzinfo=timezone.utc),
                    angebotsgeber=Geschaeftspartner(),
                    angebotsnehmer=Geschaeftspartner(),
                    unterzeichner_angebotsnehmer=Person(),
                    unterzeichner_angebotsgeber=Person(),
                    varianten=[],
                ),
                id="all attributes at first level",
            ),
        ],
    )
    def test_serialization_roundtrip(self, angebot: Angebot) -> None:
        """
        Test de-/serialisation of Angebot
        """
        assert_serialization_roundtrip(angebot)



"""
Contains Energiemix class
and corresponding marshmallow schema for de-/serialization
"""

from decimal import Decimal
from typing import Optional

from ..enum.oekolabel import Oekolabel
from ..enum.oekozertifikat import Oekozertifikat
from ..enum.sparte import Sparte
from ..utils import postprocess_docstring
from .com import COM
from .energieherkunft import Energieherkunft

# pylint: disable=too-few-public-methods, too-many-instance-attributes
# pylint: disable=no-name-in-module


@postprocess_docstring
class Energiemix(COM):
    """
    Zusammensetzung der gelieferten Energie aus den verschiedenen Primärenergieformen.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Energiemix.svg" type="image/svg+xml"></object>

    .. HINT::
        `Energiemix JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Energiemix.json>`_

    """

    #: Eindeutige Nummer zur Identifizierung des Energiemixes
    energiemixnummer: Optional[int] = None
    #: Strom oder Gas etc.
    energieart: Optional[Sparte] = None
    #: Bezeichnung des Energiemix
    bezeichnung: Optional[str] = None
    #: Jahr, für das der Energiemix gilt
    gueltigkeitsjahr: Optional[int] = None
    #: Anteile der jeweiligen Erzeugungsart
    anteil: Optional[list[Energieherkunft]] = None

    #: Bemerkung zum Energiemix
    bemerkung: Optional[str] = None
    #: Höhe des erzeugten CO2-Ausstosses in g/kWh
    co2_emission: Optional[Decimal] = None
    #: Höhe des erzeugten Atommülls in g/kWh
    atommuell: Optional[Decimal] = None
    #: Zertifikate für den Energiemix
    oekozertifikate: Optional[list[Oekozertifikat]] = None
    #: Ökolabel für den Energiemix
    oekolabel: Optional[list[Oekolabel]] = None
    #: Kennzeichen, ob der Versorger zu den Öko Top Ten gehört
    ist_in_oeko_top_ten: Optional[bool] = None
    #: Internetseite, auf der die Strommixdaten veröffentlicht sind
    website: Optional[str] = None



"""
Contains Zustaendigkeit class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

from ..enum.themengebiet import Themengebiet
from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Zustaendigkeit(COM):
    """
    Enthält die zeitliche Zuordnung eines Ansprechpartners zu Abteilungen und Zuständigkeiten.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zustaendigkeit.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zustaendigkeit JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Zustaendigkeit.json>`_

    """

    themengebiet: Optional[Themengebiet] = None
    """
    Hier kann eine thematische Zuordnung des Ansprechpartners bzw. der Person angegeben werden
    """

    position: Optional[str] = None  #: Berufliche Rolle des Ansprechpartners/ der Person
    abteilung: Optional[str] = None  #: Abteilung, in der der Ansprechpartner/ die Person tätig ist



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


# pylint: disable=line-too-long
class Geraetetyp(StrEnum):
    """
    Auflistung möglicher abzurechnender Gerätetypen.
    """

    MULTIPLEXANLAGE = "MULTIPLEXANLAGE"  #: Multiplexeranlage
    PAUSCHALANLAGE = "PAUSCHALANLAGE"  #: Pauschalanlagen
    VERSTAERKERANLAGE = "VERSTAERKERANLAGE"  #: Verstärkeranlage
    SUMMATIONSGERAET = "SUMMATIONSGERAET"  #: Summationsgerät
    IMPULSGEBER = "IMPULSGEBER"  #: Impulsgeber
    MENGENUMWERTER = "MENGENUMWERTER"  #: Mengenumwerter
    STROMWANDLER = "STROMWANDLER"  #: Stromwandler
    SPANNUNGSWANDLER = "SPANNUNGSWANDLER"  #: Spannungs-Wandler
    KOMBIMESSWANDLER = "KOMBIMESSWANDLER"  #: Kombimesswandler
    BLOCKSTROMWANDLER = "BLOCKSTROMWANDLER"  #: Blockstromwandler
    DATENLOGGER = "DATENLOGGER"  #: Datenlogger
    KOMMUNIKATIONSANSCHLUSS = "KOMMUNIKATIONSANSCHLUSS"  #: Kommunikationsanschluss
    MODEM = "MODEM"  #: Modem
    TELEKOMMUNIKATIONSEINRICHTUNG = "TELEKOMMUNIKATIONSEINRICHTUNG"  #: vom Messstellenbetreiber beigestellte Telekommunikationseinrichtung (Telefonanschluss)
    MODERNE_MESSEINRICHTUNG = "MODERNE_MESSEINRICHTUNG"  #: moderne Messeinrichtung
    INTELLIGENTES_MESSYSTEM = "INTELLIGENTES_MESSYSTEM"  #: Intelligentes Messystem
    STEUEREINRICHTUNG = "STEUEREINRICHTUNG"  #: Steuereinrichtung
    TARIFSCHALTGERAET = "TARIFSCHALTGERAET"  #: Tarifschaltgerät
    RUNDSTEUEREMPFAENGER = "RUNDSTEUEREMPFAENGER"  #: Rundsteuerempfänger
    OPTIONALE_ZUS_ZAEHLEINRICHTUNG = "OPTIONALE_ZUS_ZAEHLEINRICHTUNG"  #: optionale zusätzliche Zähleinrichtung
    MESSWANDLERSATZ_IMS_MME = "MESSWANDLERSATZ_IMS_MME"  #: Messwandlersatz Strom iMS und mME, NSP
    KOMBIMESSWANDLER_IMS_MME = "KOMBIMESSWANDLER_IMS_MME"  #: Kombimesswandlersatz (Strom u. Spg) iMS und mME
    TARIFSCHALTGERAET_IMS_MME = "TARIFSCHALTGERAET_IMS_MME"  #: Tarifschaltung iMS und mME
    RUNDSTEUEREMPFAENGER_IMS_MME = "RUNDSTEUEREMPFAENGER_IMS_MME"  #: Rundsteuerempfänger iMS und mME
    TEMPERATUR_KOMPENSATION = "TEMPERATUR_KOMPENSATION"  #: Temperaturkompensation
    HOECHSTBELASTUNGS_ANZEIGER = "HOECHSTBELASTUNGS_ANZEIGER"  #: Höchsbelastungsanzeiger
    SONSTIGES_GERAET = "SONSTIGES_GERAET"  #: Sonstiges Gerät
    EDL_21 = "EDL_21"  #: EDL21
    EDL_40_ZAEHLERAUFSATZ = "EDL_40_ZAEHLERAUFSATZ"  #: EDL 40 Zähleraufsatz für Zähler
    EDL_40 = "EDL_40"  #: EDL 40
    TELEFONANSCHLUSS = "TELEFONANSCHLUSS"  #: Telefonanschluss
    MODEM_GSM = "MODEM_GSM"  #: Modem-GSM
    MODEM_GPRS = "MODEM_GPRS"  #: Modem-GPRS
    MODEM_FUNK = "MODEM_FUNK"  #: Modem-Funk
    MODEM_GSM_O_LG = "MODEM_GSM_O_LG"  #: vom Messstellenbetreiber beigestelltes GSM-Modem ohne Lastgangmessung
    MODEM_GSM_M_LG = "MODEM_GSM_M_LG"  #: vom Messstellenbetreiber beigestelltes GSM-Modem mit Lastgangmessung
    MODEM_FESTNETZ = "MODEM_FESTNETZ"  #: vom Messstellenbetreiber beigestelltes Festnetz-Modem
    MODEM_GPRS_M_LG = "MODEM_GPRS_M_LG"  #: vom Messstellenbetreiber bereitgestelltes GPRS-Modem Lastgangmessung
    PLC_KOM = "PLC_KOM"  #: PLC-Kom.-Einrichtung (Powerline Communication)
    ETHERNET_KOM = "ETHERNET_KOM"  #: Ethernet-Kom.-Einricht. LAN/WLAN
    DSL_KOM = "DSL_KOM"  #: DSL-Kommunikationseinrichtung
    LTE_KOM = "LTE_KOM"  #: LTE-Kommunikationseinrichtung
    KOMPAKT_MU = "KOMPAKT_MU"  #: Kompaktmengenumwerter
    SYSTEM_MU = "SYSTEM_MU"  #: Systemmengenumwerter
    TEMPERATUR_MU = "TEMPERATUR_MU"  #: Temperaturmengenumwerter
    ZUSTANDS_MU = "ZUSTANDS_MU"  #: Zustandsmengenumwerter



from datetime import datetime, timezone

import pytest

from bo4e import ArithmetischeOperation, Messlokationszuordnung
from tests.serialization_helper import assert_serialization_roundtrip


class TestMesslokationszuordnung:
    @pytest.mark.parametrize(
        "messlokationszuordnung",
        [
            pytest.param(
                Messlokationszuordnung(
                    messlokations_id="DE0010688516810000000000000012345",
                    arithmetik=ArithmetischeOperation.ADDITION,
                    gueltig_seit=datetime(year=2021, month=1, day=13).replace(tzinfo=timezone.utc),
                    gueltig_bis=datetime(year=2021, month=5, day=4).replace(tzinfo=timezone.utc),
                )
            )
        ],
    )
    def test_serialization_roundtrip(self, messlokationszuordnung: Messlokationszuordnung) -> None:
        """
        Test de-/serialisation of Messlokationszuordnung.
        """

        assert_serialization_roundtrip(messlokationszuordnung)



"""
Contains TarifpreisstaffelProOrt class
and corresponding marshmallow schema for de-/serialization
"""

from decimal import Decimal
from typing import Optional

from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-few-public-methods


@postprocess_docstring
class TarifpreisstaffelProOrt(COM):
    """
    Gibt die Staffelgrenzen der jeweiligen Preise an

    .. raw:: html

        <object data="../_static/images/bo4e/com/TarifpreisstaffelProOrt.svg" type="image/svg+xml"></object>

    .. HINT::
        `TarifpreisstaffelProOrt JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/TarifpreisstaffelProOrt.json>`_

    """

    # todo: decimal doesn't make sense here imo
    # https://github.com/Hochfrequenz/BO4E-python/issues/344

    #: Der Arbeitspreis in ct/kWh
    arbeitspreis: Optional[Decimal] = None
    #: Der Arbeitspreis für Verbräuche in der Niedertarifzeit in ct/kWh
    arbeitspreis_n_t: Optional[Decimal] = None
    #: Der Grundpreis in Euro/Jahr
    grundpreis: Optional[Decimal] = None
    #: Unterer Wert, ab dem die Staffel gilt (inklusive)
    staffelgrenze_von: Optional[Decimal] = None
    #: Oberer Wert, bis zu dem die Staffel gilt (exklusive)
    staffelgrenze_bis: Optional[Decimal] = None

    # there are no optional attributes



"""
Contains Zeitraum class
and corresponding marshmallow schema for de-/serialization
"""

from decimal import Decimal
from typing import Optional

import pydantic

from ..enum.mengeneinheit import Mengeneinheit
from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Zeitraum(COM):
    """
    Diese Komponente wird zur Abbildung von Zeiträumen in Form von Dauern oder der Angabe von Start und Ende verwendet.
    Es muss daher eine der drei Möglichkeiten angegeben sein:
    - Einheit und Dauer oder
    - Zeitraum: Startdatum bis Enddatum oder
    - Zeitraum: Startzeitpunkt (Datum und Uhrzeit) bis Endzeitpunkt (Datum und Uhrzeit)

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zeitraum.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zeitraum JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Zeitraum.json>`_

    """

    einheit: Optional[Mengeneinheit] = None
    dauer: Optional[Decimal] = None
    startdatum: Optional[pydantic.AwareDatetime] = None
    enddatum: Optional[pydantic.AwareDatetime] = None
    startzeitpunkt: Optional[pydantic.AwareDatetime] = None
    endzeitpunkt: Optional[pydantic.AwareDatetime] = None



"""
Contains Messlokationszuordnung class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

import pydantic

from ..enum.arithmetische_operation import ArithmetischeOperation
from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Messlokationszuordnung(COM):
    """
    Mit dieser Komponente werden Messlokationen zu Marktlokationen zugeordnet.
    Dabei kann eine arithmetische Operation (Addition, Subtraktion, Multiplikation, Division) angegeben werden,
    mit der die Messlokation zum Verbrauch der Marktlokation beiträgt.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Messlokationszuordnung.svg" type="image/svg+xml"></object>

    .. HINT::
        `Messlokationszuordnung JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Messlokationszuordnung.json>`_

    """

    messlokations_id: Optional[str] = None
    """
    ID der zugeordneten Messlokation
    """
    arithmetik: Optional[ArithmetischeOperation] = None

    gueltig_seit: Optional[pydantic.AwareDatetime] = None
    """
    inklusives Beginndatum
    """
    gueltig_bis: Optional[pydantic.AwareDatetime] = None
    """
    exklusives Endedatum
    """



"""
Contains Rechnungsposition class and corresponding marshmallow schema for de-/serialization
"""

# pylint: disable=too-few-public-methods, too-many-instance-attributes
from typing import Optional

import pydantic

from ..enum.bdewartikelnummer import BDEWArtikelnummer
from ..enum.mengeneinheit import Mengeneinheit
from ..utils import postprocess_docstring
from .betrag import Betrag
from .com import COM
from .menge import Menge
from .preis import Preis
from .steuerbetrag import Steuerbetrag


@postprocess_docstring
class Rechnungsposition(COM):
    """
    Über Rechnungspositionen werden Rechnungen strukturiert.
    In einem Rechnungsteil wird jeweils eine in sich geschlossene Leistung abgerechnet.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Rechnungsposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Rechnungsposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Rechnungsposition.json>`_

    """

    #: Fortlaufende Nummer für die Rechnungsposition
    positionsnummer: Optional[int] = None

    lieferung_von: Optional[
        pydantic.AwareDatetime
    ] = None  #: Start der Lieferung für die abgerechnete Leistung (inklusiv)
    lieferung_bis: Optional[
        pydantic.AwareDatetime
    ] = None  #: Ende der Lieferung für die abgerechnete Leistung (exklusiv)

    #: Bezeichung für die abgerechnete Position
    positionstext: Optional[str] = None

    #: Die abgerechnete Menge mit Einheit
    positions_menge: Optional[Menge] = None
    #: Der Preis für eine Einheit der energetischen Menge
    einzelpreis: Optional[Preis] = None

    teilsumme_netto: Optional[Betrag] = None
    """
    Das Ergebnis der Multiplikation aus einzelpreis * positionsMenge * (Faktor aus zeitbezogeneMenge).
    Z.B. 12,60€ * 120 kW * 3/12 (für 3 Monate).
    """
    # the cross check in general doesn't work because Betrag and Preis use different enums to describe the currency
    # see https://github.com/Hochfrequenz/BO4E-python/issues/126

    #: Auf die Position entfallende Steuer, bestehend aus Steuersatz und Betrag
    teilsumme_steuer: Optional[Steuerbetrag] = None

    #: Falls sich der Preis auf eine Zeit bezieht, steht hier die Einheit
    zeiteinheit: Optional[Mengeneinheit] = None

    #: Kennzeichnung der Rechnungsposition mit der Standard-Artikelnummer des BDEW
    artikelnummer: Optional[BDEWArtikelnummer] = None
    #: Marktlokation, die zu dieser Position gehört
    lokations_id: Optional[str] = None

    zeitbezogene_menge: Optional[Menge] = None
    """
    Eine auf die Zeiteinheit bezogene Untermenge.
    Z.B. bei einem Jahrespreis, 3 Monate oder 146 Tage.
    Basierend darauf wird der Preis aufgeteilt.
    """
    #: Nettobetrag für den Rabatt dieser Position
    teilrabatt_netto: Optional[Betrag] = None

    #: Standardisierte vom BDEW herausgegebene Liste, welche im Strommarkt die BDEW-Artikelnummer ablöst
    artikel_id: Optional[str] = None



"""
Contains Zaehler class
and corresponding marshmallow schema for de-/serialization
"""

from decimal import Decimal

# pylint: disable=unused-argument
# pylint: disable=no-name-in-module
from typing import Annotated, Optional

import pydantic
from pydantic import Field

from ..bo.geraet import Geraet
from ..com.zaehlwerk import Zaehlwerk
from ..enum.befestigungsart import Befestigungsart
from ..enum.messwerterfassung import Messwerterfassung
from ..enum.registeranzahl import Registeranzahl
from ..enum.sparte import Sparte
from ..enum.typ import Typ
from ..enum.zaehlerauspraegung import Zaehlerauspraegung
from ..enum.zaehlergroesse import Zaehlergroesse
from ..enum.zaehlertyp import Zaehlertyp
from ..enum.zaehlertypspezifikation import ZaehlertypSpezifikation
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt
from .geschaeftspartner import Geschaeftspartner

# pylint: disable=too-many-instance-attributes, too-few-public-methods


@postprocess_docstring
class Zaehler(Geschaeftsobjekt):
    """
    Object containing information about a meter/"Zaehler".

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Zaehler.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zaehler JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Zaehler.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.ZAEHLER
    zaehlernummer: Optional[str] = None  #: Nummerierung des Zählers,vergeben durch den Messstellenbetreiber
    sparte: Optional[Sparte] = None  #: Strom oder Gas
    zaehlerauspraegung: Optional[Zaehlerauspraegung] = None  #: Spezifikation die Richtung des Zählers betreffend
    zaehlertyp: Optional[Zaehlertyp] = None  #: Typisierung des Zählers
    zaehlwerke: Optional[list[Zaehlwerk]] = None
    registeranzahl: Optional[Registeranzahl] = None  #: Spezifikation bezüglich unterstützter Tarif
    zaehlerkonstante: Optional[Decimal] = None  #: Zählerkonstante auf dem Zähler
    eichung_bis: Optional[pydantic.AwareDatetime] = None  #: Bis zu diesem Datum (exklusiv) ist der Zähler geeicht.
    letzte_eichung: Optional[
        pydantic.AwareDatetime
    ] = None  #: Zu diesem Datum fand die letzte Eichprüfung des Zählers statt.
    zaehlerhersteller: Optional[Geschaeftspartner] = None  #: Der Hersteller des Zählers
    ist_fernschaltbar: Optional[bool] = None  #: Fernschaltung
    messwerterfassung: Optional[Messwerterfassung] = None  #: Messwerterfassung des Zählers
    zaehlertypSpezifikation: Optional[ZaehlertypSpezifikation] = None  #: Besondere Spezifikation des Zählers
    befestigungsart: Optional[Befestigungsart] = None  #: Befestigungsart
    zaehlergroesse: Optional[Zaehlergroesse] = None  #: Größe des Zählers
    geraete: Optional[list[Geraet]] = None  #: Liste der Geräte, die zu diesem Zähler gehören, bspw. Smartmeter-Gateway



"""
Contains Messlokation class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Annotated, Optional

from pydantic import Field

from ..bo.geraet import Geraet
from ..com.adresse import Adresse
from ..com.dienstleistung import Dienstleistung
from ..com.geokoordinaten import Geokoordinaten
from ..com.katasteradresse import Katasteradresse
from ..enum.netzebene import Netzebene
from ..enum.sparte import Sparte
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt
from .zaehler import Zaehler

# pylint: disable=too-many-instance-attributes, too-few-public-methods


@postprocess_docstring
class Messlokation(Geschaeftsobjekt):
    """
    Object containing information about a Messlokation

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Messlokation.svg" type="image/svg+xml"></object>

    .. HINT::
        `Messlokation JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Messlokation.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.MESSLOKATION
    #: Die Messlokations-Identifikation; Das ist die frühere Zählpunktbezeichnung
    messlokations_id: Optional[str] = None
    #: Sparte der Messlokation, z.B. Gas oder Strom
    sparte: Optional[Sparte] = None

    #: Spannungsebene der Messung
    netzebene_messung: Optional[Netzebene] = None
    #: Die Nummer des Messgebietes in der ene't-Datenbank
    messgebietnr: Optional[str] = None
    #: Liste der Geräte, die zu dieser Messstelle gehört
    geraete: Optional[list[Geraet]] = None
    #: Liste der Messdienstleistungen, die zu dieser Messstelle gehört
    messdienstleistung: Optional[list[Dienstleistung]] = None  # todo: rename to plural
    #: Zähler, die zu dieser Messlokation gehören
    messlokationszaehler: Optional[list[Zaehler]] = None

    # only one of the following two optional codenr attributes can be set
    grundzustaendiger_msb_codenr: Optional[str] = None
    """
    Codenummer des grundzuständigen Messstellenbetreibers, der für diese Messlokation zuständig ist.
    (Dieser ist immer dann Messstellenbetreiber, wenn kein anderer MSB die Einrichtungen an der Messlokation betreibt.)
    """
    grundzustaendiger_msbim_codenr: Optional[str] = None
    """
    Codenummer des grundzuständigen Messstellenbetreibers für intelligente Messsysteme, der für diese Messlokation
    zuständig ist.
    (Dieser ist immer dann Messstellenbetreiber, wenn kein anderer MSB die Einrichtungen an der Messlokation betreibt.)
    """
    # only one of the following three optional address attributes can be set
    messadresse: Optional[Adresse] = None
    """
    Die Adresse, an der die Messeinrichtungen zu finden sind.
    (Nur angeben, wenn diese von der Adresse der Marktlokation abweicht.)
    """
    geoadresse: Optional[Geokoordinaten] = None
    """
    Alternativ zu einer postalischen Adresse kann hier ein Ort mittels Geokoordinaten angegeben werden
    (z.B. zur Identifikation von Sendemasten).
    """
    katasterinformation: Optional[Katasteradresse] = None
    """
    Alternativ zu einer postalischen Adresse und Geokoordinaten kann hier eine Ortsangabe mittels Gemarkung und
    Flurstück erfolgen.
    """



"""
Contains StandorteigenschaftenStrom class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Optional

from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-few-public-methods


@postprocess_docstring
class StandorteigenschaftenStrom(COM):
    """
    Standorteigenschaften der Sparte Strom

    .. raw:: html

        <object data="../_static/images/bo4e/com/StandorteigenschaftenStrom.svg" type="image/svg+xml"></object>

    .. HINT::
        `StandorteigenschaftenStrom JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/StandorteigenschaftenStrom.json>`_

    """

    #: Die EIC-Nummer des Bilanzierungsgebietes
    bilanzierungsgebiet_eic: Optional[str] = None
    # todo: use EIC validation: https://github.com/Hochfrequenz/BO4E-python/issues/147

    #: Der Name der Regelzone
    regelzone: Optional[str] = None

    #: De EIC-Nummer der Regelzone
    regelzone_eic: Optional[str] = None
    # todo: use EIC validation: https://github.com/Hochfrequenz/BO4E-python/issues/147



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Verwendungszweck(StrEnum):
    """
    Verwendungszweck der Werte Marktlokation
    """

    NETZNUTZUNGSABRECHNUNG = "NETZNUTZUNGSABRECHNUNG"  #: Netznutzungsabrechnung
    BILANZKREISABRECHNUNG = "BILANZKREISABRECHNUNG"  #: Bilanzkreisabrechnung
    MEHRMINDERMENGENABRECHNUNG = "MEHRMINDERMENGENABRECHNUNG"  #: Mehrmindermengenabrechnung
    ENDKUNDENABRECHNUNG = "ENDKUNDENABRECHNUNG"  #: Endkundenabrechnung
    UEBERMITTLUNG_AN_DAS_HKNR = "UEBERMITTLUNG_AN_DAS_HKNR"  #: Übermittlung an das Herkunftsnachweisregister (HKNR)
    #: Zur Ermittlung der Ausgeglichenheit von Bilanzkreisen
    ERMITTLUNG_AUSGEGLICHENHEIT_BILANZKREIS = "ERMITTLUNG_AUSGEGLICHENHEIT_BILANZKREIS"



"""
Contains Energiemenge class
and corresponding marshmallow schema for de-/serialization
"""

from typing import Annotated, Optional

from pydantic import Field

from ..com.verbrauch import Verbrauch
from ..enum.lokationstyp import Lokationstyp
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module


@postprocess_docstring
class Energiemenge(Geschaeftsobjekt):
    """
    Abbildung von Mengen, die Lokationen zugeordnet sind

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Energiemenge.svg" type="image/svg+xml"></object>

    .. HINT::
        `Energiemenge JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Energiemenge.json>`_

    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.ENERGIEMENGE
    #: Eindeutige Nummer der Marktlokation bzw. der Messlokation, zu der die Energiemenge gehört
    lokations_id: Optional[str] = None
    # todo: add validator such that only mess- or marktlokations IDs are accepted + cross check with lokationstyp
    #: Gibt an, ob es sich um eine Markt- oder Messlokation handelt
    lokationstyp: Optional[Lokationstyp] = None

    #: Gibt den Verbrauch in einer Zeiteinheit an
    energieverbrauch: Optional[list[Verbrauch]] = None
    # there are no optional attributes



import pytest

from bo4e import Medium, Mengeneinheit, Messart, Messgroesse, Wertermittlungsverfahren, Zeitreihe
from tests.serialization_helper import assert_serialization_roundtrip


class TestZeitreihe:
    @pytest.mark.parametrize(
        "zeitreihe",
        [
            pytest.param(
                Zeitreihe(
                    bezeichnung="Foo",
                    beschreibung="Bar",
                    version="0.0.1",
                    messgroesse=Messgroesse.BLINDLEISTUNG,
                    messart=Messart.MAXIMALWERT,
                    medium=Medium.STROM,
                    einheit=Mengeneinheit.KVARH,
                    wertherkunft=Wertermittlungsverfahren.MESSUNG,
                    werte=[],
                )
            ),
        ],
    )
    def test_serialization_roundtrip(self, zeitreihe: Zeitreihe) -> None:
        assert_serialization_roundtrip(zeitreihe)



"""
Contains Angebot class and corresponding marshmallow schema for de-/serialization
"""

# pylint: disable=too-few-public-methods, too-many-instance-attributes
# pylint: disable=no-name-in-module
from typing import Annotated, Optional

import pydantic
from pydantic import Field

from ..com.angebotsvariante import Angebotsvariante
from ..enum.sparte import Sparte
from ..enum.typ import Typ
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt
from .geschaeftspartner import Geschaeftspartner
from .person import Person


@postprocess_docstring
class Angebot(Geschaeftsobjekt):
    """
    Mit diesem BO kann ein Versorgungsangebot zur Strom- oder Gasversorgung oder die Teilnahme an einer Ausschreibung
    übertragen werden. Es können verschiedene Varianten enthalten sein (z.B. ein- und mehrjährige Laufzeit).
    Innerhalb jeder Variante können Teile enthalten sein, die jeweils für eine oder mehrere Marktlokationen erstellt
    werden.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Angebot.svg" type="image/svg+xml"></object>

    .. HINT::
        `Angebot JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Angebot.json>`_


    """

    typ: Annotated[Optional[Typ], Field(alias="_typ")] = Typ.ANGEBOT
    #: Eindeutige Nummer des Angebotes
    angebotsnummer: Optional[str] = None
    #: Erstellungsdatum des Angebots
    angebotsdatum: Optional[pydantic.AwareDatetime] = None
    #: Sparte, für die das Angebot abgegeben wird (Strom/Gas)
    sparte: Optional[Sparte] = None
    #: Ersteller des Angebots
    angebotsgeber: Optional[Geschaeftspartner] = None
    #: Empfänger des Angebots
    angebotsnehmer: Optional[Geschaeftspartner] = None

    varianten: Optional[list[Angebotsvariante]] = None
    """ Eine oder mehrere Varianten des Angebots mit den Angebotsteilen;
    Ein Angebot besteht mindestens aus einer Variante."""

    anfragereferenz: Optional[str] = None
    """	Referenz auf eine Anfrage oder Ausschreibung;
    Kann dem Empfänger des Angebotes bei Zuordnung des Angebotes zur Anfrage bzw. Ausschreibung helfen."""
    #: Bis zu diesem Zeitpunkt (Tag/Uhrzeit) inklusive gilt das Angebot
    bindefrist: Optional[pydantic.AwareDatetime] = None
    #: Person, die als Angebotsnehmer das Angebot angenommen hat
    unterzeichner_angebotsnehmer: Optional[Person] = None
    #: Person, die als Angebotsgeber das Angebots ausgestellt hat
    unterzeichner_angebotsgeber: Optional[Person] = None



"""
Contains Preisposition class and corresponding marshmallow schema for de-/serialization
"""

from decimal import Decimal

# pylint: disable=no-name-in-module
from typing import Optional

from ..enum.bdewartikelnummer import BDEWArtikelnummer
from ..enum.bemessungsgroesse import Bemessungsgroesse
from ..enum.kalkulationsmethode import Kalkulationsmethode
from ..enum.leistungstyp import Leistungstyp
from ..enum.mengeneinheit import Mengeneinheit
from ..enum.tarifzeit import Tarifzeit
from ..enum.waehrungseinheit import Waehrungseinheit
from ..utils import postprocess_docstring
from .com import COM
from .preisstaffel import Preisstaffel

# pylint: disable=too-few-public-methods, too-many-instance-attributes


@postprocess_docstring
class Preisposition(COM):
    """
    Preis für eine definierte Lieferung oder Leistung innerhalb eines Preisblattes

    .. raw:: html

        <object data="../_static/images/bo4e/com/Preisposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Preisposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Preisposition.json>`_

    """

    #: Das Modell, das der Preisbildung zugrunde liegt
    berechnungsmethode: Optional[Kalkulationsmethode] = None
    #: Standardisierte Bezeichnung für die abgerechnete Leistungserbringung
    leistungstyp: Optional[Leistungstyp] = None  #
    #: Bezeichnung für die in der Position abgebildete Leistungserbringung
    leistungsbezeichnung: Optional[str] = None
    #: Festlegung, mit welcher Preiseinheit abgerechnet wird, z.B. Ct. oder €
    preiseinheit: Optional[Waehrungseinheit] = None
    #: Hier wird festgelegt, auf welche Bezugsgrösse sich der Preis bezieht, z.B. kWh oder Stück
    bezugsgroesse: Optional[Mengeneinheit] = None
    #: Preisstaffeln, die zu dieser Preisposition gehören
    preisstaffeln: Optional[list[Preisstaffel]] = None

    zeitbasis: Optional[Mengeneinheit] = None
    """
    Die Zeit(dauer) auf die sich der Preis bezieht.
    Z.B. ein Jahr für einen Leistungspreis der in €/kW/Jahr ausgegeben wird
    """
    #: Festlegung, für welche Tarifzeit der Preis hier festgelegt ist
    tarifzeit: Optional[Tarifzeit] = None
    bdew_artikelnummer: Optional[BDEWArtikelnummer] = None
    """
    Eine vom BDEW standardisierte Bezeichnug für die abgerechnete Leistungserbringung;
    Diese Artikelnummer wird auch im Rechnungsteil der INVOIC verwendet.
    """
    #: Mit der Menge der hier angegebenen Größe wird die Staffelung/Zonung durchgeführt. Z.B. Vollbenutzungsstunden
    zonungsgroesse: Optional[Bemessungsgroesse] = None
    #: Der Anteil der Menge der Blindarbeit in Prozent von der Wirkarbeit, für die keine Abrechnung erfolgt
    freimenge_blindarbeit: Optional[Decimal] = None
    freimenge_leistungsfaktor: Optional[Decimal] = None
    """
    Der cos phi (Verhältnis Wirkleistung/Scheinleistung) aus dem die Freimenge für die Blindarbeit berechnet wird als
    tan phi (Verhältnis Blindleistung/Wirkleistung)
    """
    gruppenartikel_id: Optional[str] = None
    """
    Übergeordnete Gruppen-ID, die sich ggf. auf die Artikel-ID in der Preisstaffel bezieht
    """



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Waehrungscode(StrEnum):
    """
    Aufzählung der Währungscodes.
    """

    AED = "AED"  #: UAE Dirham
    AFN = "AFN"  #: Afghani
    ALL = "ALL"  #: Lek
    AMD = "AMD"  #: Armenian Dram
    ANG = "ANG"  #: Netherlands Antillean Guilder
    AOA = "AOA"  #: Kwanza
    ARS = "ARS"  #: Argentine Peso
    AUD = "AUD"  #: Australian Dollar
    AWG = "AWG"  #: Aruban Florin
    AZN = "AZN"  #: Azerbaijanian Manat
    BAM = "BAM"  #: Convertible Mark
    BBD = "BBD"  #: Barbados Dollar
    BDT = "BDT"  #: Taka
    BGN = "BGN"  #: Bulgarian Lev
    BHD = "BHD"  #: Bahraini Dinar
    BIF = "BIF"  #: Burundi Franc
    BMD = "BMD"  #: Bermudian Dollar
    BND = "BND"  #: Brunei Dollar
    BOB = "BOB"  #: Boliviano
    BOV = "BOV"  #: Mvdol
    BRL = "BRL"  #: Brazilian Real
    BSD = "BSD"  #: Bahamian Dollar
    BTN = "BTN"  #: Ngultrum
    BWP = "BWP"  #: Pula
    BYN = "BYN"  #: Belarusian Ruble
    BYR = "BYR"  #: Belarusian Ruble
    BZD = "BZD"  #: Belize Dollar
    CAD = "CAD"  #: Canadian Dollar
    CDF = "CDF"  #: Congolese Franc
    CHE = "CHE"  #: WIR Euro
    CHF = "CHF"  #: Swiss Franc
    CHW = "CHW"  #: WIR Franc
    CLF = "CLF"  #: Unidad de Fomento
    CLP = "CLP"  #: Chilean Peso
    CNY = "CNY"  #: Yuan Renminbi
    COP = "COP"  #: Colombian Peso
    COU = "COU"  #: Unidad de Valor Real
    CRC = "CRC"  #: Costa Rican Colon
    CUC = "CUC"  #: Peso Convertible
    CUP = "CUP"  #: Cuban Peso
    CVE = "CVE"  #: Cape Verde Escudo
    CZK = "CZK"  #: Czech Koruna
    DJF = "DJF"  #: Djibouti Franc
    DKK = "DKK"  #: Danish Krone
    DOP = "DOP"  #: Dominican Peso
    DZD = "DZD"  #: Algerian Dinar
    EGP = "EGP"  #: Egyptian Pound
    ERN = "ERN"  #: Nakfa
    ETB = "ETB"  #: Ethiopian Birr
    EUR = "EUR"  #: Euro
    FJD = "FJD"  #: Fiji Dollar
    FKP = "FKP"  #: Falkland Islands Pound
    GBP = "GBP"  #: Pound Sterling
    GEL = "GEL"  #: Lari
    GHS = "GHS"  #: Ghana Cedi
    GIP = "GIP"  #: Gibraltar Pound
    GMD = "GMD"  #: Dalasi
    GNF = "GNF"  #: Guinea Franc
    GTQ = "GTQ"  #: Quetzal
    GYD = "GYD"  #: Guyana Dollar
    HKD = "HKD"  #: Hong Kong Dollar
    HNL = "HNL"  #: Lempira
    HRK = "HRK"  #: Croatian Kuna
    HTG = "HTG"  #: Gourde
    HUF = "HUF"  #: Forint
    IDR = "IDR"  #: Rupiah
    ILS = "ILS"  #: New Israeli Sheqel
    INR = "INR"  #: Indian Rupee
    IQD = "IQD"  #: Iraqi Dinar
    IRR = "IRR"  #: Iranian Rial
    ISK = "ISK"  #: Iceland Krona
    JMD = "JMD"  #: Jamaican Dollar
    JOD = "JOD"  #: Jordanian Dinar
    JPY = "JPY"  #: Yen
    KES = "KES"  #: Kenyan Shilling
    KGS = "KGS"  #: Som
    KHR = "KHR"  #: Riel
    KMF = "KMF"  #: Comoro Franc
    KPW = "KPW"  #: North Korean Won
    KRW = "KRW"  #: Won
    KWD = "KWD"  #: Kuwaiti Dinar
    KYD = "KYD"  #: Cayman Islands Dollar
    KZT = "KZT"  #: Tenge
    LAK = "LAK"  #: Kip
    LBP = "LBP"  #: Lebanese Pound
    LKR = "LKR"  #: Sri Lanka Rupee
    LRD = "LRD"  #: Liberian Dollar
    LSL = "LSL"  #: Loti
    LTL = "LTL"  #: Lithuanian Litas
    LYD = "LYD"  #: Libyan Dinar
    MAD = "MAD"  #: Moroccan Dirham
    MDL = "MDL"  #: Moldovan Leu
    MGA = "MGA"  #: Malagasy Ariary
    MKD = "MKD"  #: Denar
    MMK = "MMK"  #: Kyat
    MNT = "MNT"  #: Tugrik
    MOP = "MOP"  #: Pataca
    MRO = "MRO"  #: Ouguiya
    MUR = "MUR"  #: Mauritius Rupee
    MVR = "MVR"  #: Rufiyaa
    MWK = "MWK"  #: Kwacha
    MXN = "MXN"  #: Mexican Peso
    MXV = "MXV"  #: Mexican Unidad de Inversion (UDI)
    MYR = "MYR"  #: Malaysian Ringgit
    MZN = "MZN"  #: Mozambique Metical
    NAD = "NAD"  #: Namibia Dollar
    NGN = "NGN"  #: Naira
    NIO = "NIO"  #: Cordoba Oro
    NOK = "NOK"  #: Norwegian Krone
    NPR = "NPR"  #: Nepalese Rupee
    NZD = "NZD"  #: New Zealand Dollar
    OMR = "OMR"  #: Rial Omani
    PAB = "PAB"  #: Balboa
    PEN = "PEN"  #: Nuevo Sol
    PGK = "PGK"  #: Kina
    PHP = "PHP"  #: Philippine Peso
    PKR = "PKR"  #: Pakistan Rupee
    PLN = "PLN"  #: Zloty
    PYG = "PYG"  #: Guarani
    QAR = "QAR"  #: Qatari Rial
    RON = "RON"  #: New Romanian Leu
    RSD = "RSD"  #: Serbian Dinar
    RUB = "RUB"  #: Russian Ruble
    RUR = "RUR"  #: Russian Ruble
    RWF = "RWF"  #: Rwanda Franc
    SAR = "SAR"  #: Saudi Riyal
    SBD = "SBD"  #: Solomon Islands Dollar
    SCR = "SCR"  #: Seychelles Rupee
    SDG = "SDG"  #: Sudanese Pound
    SEK = "SEK"  #: Swedish Krona
    SGD = "SGD"  #: Singapore Dollar
    SHP = "SHP"  #: Saint Helena Pound
    SLL = "SLL"  #: Leone
    SOS = "SOS"  #: Somali Shilling
    SRD = "SRD"  #: Surinam Dollar
    SSP = "SSP"  #: South Sudanese Pound
    STD = "STD"  #: Dobra
    SVC = "SVC"  #: El Salvador Colon
    SYP = "SYP"  #: Syrian Pound
    SZL = "SZL"  #: Lilangeni
    THB = "THB"  #: Baht
    TJS = "TJS"  #: Somoni
    TMT = "TMT"  #: Turkmenistan New Manat
    TND = "TND"  #: Tunisian Dinar
    TOP = "TOP"  #: Paʻanga
    TRY = "TRY"  #: Turkish Lira
    TTD = "TTD"  #: Trinidad and Tobago Dollar
    TWD = "TWD"  #: New Taiwan Dollar
    TZS = "TZS"  #: Tanzanian Shilling
    UAH = "UAH"  #: Hryvnia
    UGX = "UGX"  #: Uganda Shilling
    USD = "USD"  #: US Dollar
    USN = "USN"  #: US Dollar (Next day)
    USS = "USS"  #: US Dollar (Same day)
    UYI = "UYI"  #: Uruguay Peso en Unidades Indexadas (URUIURUI)
    UYU = "UYU"  #: Peso Uruguayo
    UZS = "UZS"  #: Uzbekistan Sum
    VEF = "VEF"  #: Bolivar
    VND = "VND"  #: Dong
    VUV = "VUV"  #: Vatu
    WST = "WST"  #: Tala
    XAF = "XAF"  #: CFA Franc BEAC
    XAG = "XAG"  #: Silver
    XAU = "XAU"  #: Gold
    XBA = "XBA"  #: Bond Markets Unit European Composite Unit (EURCO)
    XBB = "XBB"  #: Bond Markets Unit European Monetary Unit (E.M.U.-6)
    XBC = "XBC"  #: Bond Markets Unit European Unit of Account 9 (E.U.A.-9)
    XBD = "XBD"  #: Bond Markets Unit European Unit of Account 17 (E.U.A.-17)
    XCD = "XCD"  #: East Caribbean Dollar
    XDR = "XDR"  #: SDR (Special Drawing Right)
    XOF = "XOF"  #: CFA Franc BCEAO
    XPD = "XPD"  #: Palladium
    XPF = "XPF"  #: CFP Franc
    XPT = "XPT"  #: Platinum
    XSU = "XSU"  #: Sucre
    XTS = "XTS"  #: Codes specifically reserved for testing purposes
    XUA = "XUA"  #: ADB Unit of Account
    XXX = "XXX"  #: The codes assigned for transactions where no currency ist involved
    YER = "YER"  #: Yemeni Rial
    ZAR = "ZAR"  #: Rand
    ZMW = "ZMW"  #: Zambian Kwacha
    ZWL = "ZWL"  #: Zimbabwe Dollar
